/*
 * An emulator, visualiser of the steps of an arrangement of LLFSMs from the output
 * a trace on nuXmv or NuSMV
 * (c) 2020 Vlad Estivill-Castro
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see www.gnu.org/licenses.
 * @see<a href="https://www.gnu.org/licenses"> https://www.gnu.org/licenses </a>.
 */
package emulatenuxmvtracesofllfsms;

import java.util.ArrayList;
import java.util.Scanner;
import java.util.Iterator;
import java.io.File;
import java.io.IOException;
import java.io.FileNotFoundException;

/**
 * GUI libraries
 */
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JFileChooser;
import javax.swing.UIManager;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.BoxLayout;


/*
* EMF libraries
*/
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

/*
*  Metamodel Creations
*/

import newGITmetamodelwithtypes.AdditionExpression;
import newGITmetamodelwithtypes.AndExpression;
import newGITmetamodelwithtypes.ArithmeticExpression;
import newGITmetamodelwithtypes.Arrangement;
import newGITmetamodelwithtypes.AssignmentBooleanLHS;
import newGITmetamodelwithtypes.AssignmentIntegerLHS;
import newGITmetamodelwithtypes.BooleanConstant;
import newGITmetamodelwithtypes.BooleanExpression;
import newGITmetamodelwithtypes.BooleanVariable;
import newGITmetamodelwithtypes.BooleanVariableReference;
import newGITmetamodelwithtypes.EqualExpression;
import newGITmetamodelwithtypes.IntegerConstant;
import newGITmetamodelwithtypes.IntegerVariable;
import newGITmetamodelwithtypes.IntegerVariableReference;
import newGITmetamodelwithtypes.LessThanExpression;
import newGITmetamodelwithtypes.ListOfBooleanVariables;
import newGITmetamodelwithtypes.ListOfVariables;
import newGITmetamodelwithtypes.NotExpression;
import newGITmetamodelwithtypes.OrExpression;
import newGITmetamodelwithtypes.OutputLine;
import newGITmetamodelwithtypes.PredefinedIntegerConstant;
import newGITmetamodelwithtypes.StateMachine;
import newGITmetamodelwithtypes.Statement;
import newGITmetamodelwithtypes.SubtractionExpression;
import newGITmetamodelwithtypes.TimeExpression;
import newGITmetamodelwithtypes.Transition;
import newGITmetamodelwithtypes.State;


/**
 *
 * @author vlad estivill-castro
 */
public class EmulatenuXmvTracesOfLLFSms extends javax.swing.JFrame {
    
    /**
     * The attributes of this class are mainly GUI widgets
     */
    
    /*
    * GUI Controls
    */
		
private javax.swing.JFileChooser OpenMachineFileChooser;

private javax.swing.JLabel jLabelArrangementFileName;
private javax.swing.JLabel jLabelNusmvFileName;
private javax.swing.JTextField jTextFieldArrangementName;
private javax.swing.JTextField jTextFieldTraceFile;
private javax.swing.JButton jButtonNext;
private javax.swing.JLabel jLabelNusmvKripkeState;
private javax.swing.JTextField jTextFieldKripkeState;

private javax.swing.JMenuBar jMenuBarTop;
private javax.swing.JMenu jMenuFile;
private javax.swing.JMenu jMenuAction;
private javax.swing.JMenuItem jMenuItemOpen;
private javax.swing.JMenuItem jMenuItemOpenNuSSMV;
private javax.swing.JMenuItem jMenuItemExit;
private javax.swing.JMenuItem jMenuItemNext;
private javax.swing.JMenuItem jMenuItemRedraw;
private javax.swing.JMenuItem jMenuExpandViewLLFSM;
//private javax.swing.JScrollPane jScrollPaneView;
private javax.swing.JPanel jPanelDisplayVisual;


private javax.swing.JPanel jPanelInformation;
private javax.swing.JTabbedPane jPanelMachinesArea;
private javax.swing.JPanel jPanelInforMachinesArea;
private javax.swing.JPanel[] jPanelMachines;
private javax.swing.JLabel[] jLabelMachineNames;
private javax.swing.JTextField[] jTextFieldMachineTurnNumber;
private javax.swing.JLabel[] jLabelMachineLabelTurnNumber;
private javax.swing.JPanel[] jPanelMachinesTurnAndNameInfo;
private VisualPanelForMachine[] jPanelVisualMachines;

private javax.swing.JPanel JPanelBooleanSensorVariables;
private javax.swing.JPanel JPanelIntegerSensorVariables;
private javax.swing.JPanel JPanelBooleanExternalVariables;
private javax.swing.JPanel JPanelIntegerExternalVariables;
private javax.swing.JPanel JPanelBooleanWhiteboardVariables;
private javax.swing.JPanel JPanelIntegerWhiteboardVariables;

private javax.swing.JLabel jLabelBooleanSensorVariables;
private javax.swing.JLabel jLabelIntegerSensorVariables;
private javax.swing.JLabel jLabelBooleanExternalVariables;
private javax.swing.JLabel jLabelIntegerExternalVariables;
private javax.swing.JLabel jLabelBooleanWhiteboardVariables;
private javax.swing.JLabel jLabelIntegerWhiteboardVariables;

private javax.swing.JLabel[] jLabelBooleanSensorVariablesNames;
private javax.swing.JLabel[] jLabelIntegerSensorVariablesNames;
private javax.swing.JLabel[] jLabelBooleanExternalVariablesNames;
private javax.swing.JLabel[] jLabelIntegerExternalVariablesNames;
private javax.swing.JLabel[] jLabelBooleanWhiteboardVariablesNames;
private javax.swing.JLabel[] jLabelIntegerWhiteboardVariablesNames;

private javax.swing.JTextField[] JTextFieldBooleanSensorVariablesValues;
private javax.swing.JTextField[] JTextFieldIntegerSensorVariablesValues;
private javax.swing.JTextField[] JTextFieldBooleanExternalVariablesValues;
private javax.swing.JTextField[] JTextFieldIntegerExternalVariablesValues;
private javax.swing.JTextField[] JTextFieldBooleanWhiteboardVariablesValues;
private javax.swing.JTextField[] JTextFieldIntegerWhiteboardVariablesValues;


private ArrayList<javax.swing.JLabel>[] jLabelIntegerLocalVariablesNames;
private ArrayList<javax.swing.JLabel>[] jLabelBooleanLocalVariablesNames;

private  ArrayList<javax.swing.JTextField>[] JTextFieldIntegerLocalVariablesValues;
private  ArrayList<javax.swing.JTextField>[] JTextFieldBooleanLocalVariablesValues;

private javax.swing.JPanel[] jLabelBooleanLocalVariablesPanel;
private javax.swing.JPanel[] jLabelIntegerLocalVariablesPanel;

//Color originalColor = JTEXTFIELD.getBackground();
private final Color goodFieldColor = UIManager.getColor("TextField.background");
   
    
/**
* Globals variables manipulated by the  event handlers
*/
Arrangement theArrangemant;
int totalPredicatesAfter;  // the necessary timers
Scanner scanner ;  // to read the Nusmv Trace
		// we already read the label of the Kripke state;
		// but the pressing of NEXT will perform a read of the new Kripke State variables and 
		// the label of the following one
String IDofKripkeStateAboutToBeRead; 
int theCurrentTurn;

/**
***************************************
*  The Helper functions to the Handlers
*/

    private void buildVisualPanelForTimer(
        VisualPanelForMachine thePanel,
        String aMachineName,
        String aSourceStateName,
        String aTrangetStatename,
        int aTransitionID,
        int aBound,
        int aStep,
        int aPosition ) {
                //Timer_DoorController_Locked_Opening_Pressed_16
                String [] stateNamesInVisual = new String[5];

                stateNamesInVisual[0]="dInit"+aMachineName+"_"+aSourceStateName+"_"+aTrangetStatename+"_"+aTransitionID+" (0) ";
                stateNamesInVisual[1]="Initialise (1) \nLocalCurrentTime=0;\nfinished=FALSE";
                stateNamesInVisual[2]="SetTimeBound (2) \nLocalCurrentTime="+aBound;
                stateNamesInVisual[3]="TickTime (3) \nfinished=(LocalCurrentTime<1)";
                stateNamesInVisual[4]="Decrement (4) \nLocalCurrentTime=LocalCurrentTime-"+aStep+";";

                   thePanel.AddState(stateNamesInVisual[0],"");
                   thePanel.AddState(stateNamesInVisual[1],"");
                   thePanel.AddState(stateNamesInVisual[2],"");
                   thePanel.AddState(stateNamesInVisual[3],"");
                   thePanel.AddState(stateNamesInVisual[4],"");

                   thePanel.AddTransition(stateNamesInVisual[0],stateNamesInVisual[1],
                                   "true ("+aPosition+",0)"
                                   );
                   thePanel.AddTransition(stateNamesInVisual[1],stateNamesInVisual[2],
                                   "active ("+aPosition+",1)"
                                   );
                   thePanel.AddTransition(stateNamesInVisual[2],stateNamesInVisual[3],
                                   "true ("+aPosition+",2)"
                                   );
                   thePanel.AddTransition(stateNamesInVisual[3],stateNamesInVisual[1],
                                   "finished AND (NOT active) ("+aPosition+",3)"
                                   );
                   thePanel.AddTransition(stateNamesInVisual[3],stateNamesInVisual[4],
                                   "(NOT finished) ("+aPosition+",4)"
                                   );
                   thePanel.AddTransition(stateNamesInVisual[4],stateNamesInVisual[3],
                                   "true ("+aPosition+",5)"
                                   );
    } // buildVisualPanelForTimer


    private String getBoundInAfter( BooleanExpression e) {
        System.err.println(e.getClass().getSimpleName());
        if (e.getClass().getSimpleName().contains("Constant") )
            {       BooleanConstant theConstant = (BooleanConstant) e;
                                if (theConstant.isABooleanValue()) {return "";  }
                                else return "";
            }//a constant
        else
            {   if (e.getClass().getSimpleName().contains("Variable") ) {     return "";}
                else
                    {   if (e.getClass().getSimpleName().contains("Not") )
                           {  NotExpression theNotExpression = (NotExpression) e;
                                   return
                                                  getBoundInAfter(  theNotExpression.getTheExpression());
                           }
                        else {   if (e.getClass().getSimpleName().contains("Equal") )
                                             {
                                                return "";
                                              }
                                    else { if (e.getClass().getSimpleName().contains("AndExpressionImpl") )
                                        {      AndExpression andExpression =(AndExpression) e;
                                           if (findTotalPredciatedAfter (andExpression.getLeftExpression() ) >0 )
                                                   {  return getBoundInAfter( andExpression.getLeftExpression() );}
                                           else
                                           {  return getBoundInAfter( andExpression.getRightExpression() );}
                                    }else { if (e.getClass().getSimpleName().contains("OrExpressionImpl") )
                                    {  OrExpression orExpression =(OrExpression) e;
                                       if (findTotalPredciatedAfter (orExpression.getLeftExpression() ) >0 )
                                            {  return getBoundInAfter( orExpression.getLeftExpression() );}
                                       else
                                       {  return getBoundInAfter( orExpression.getRightExpression() );}
                                      }else {  if (e.getClass().getSimpleName().contains("LessThanExpressionImpl") ) {
                                                return "";
                                               } // not Less Than
                                                  else
                                               {  if (e.getClass().getSimpleName().contains("TimeExpressionImpl") ) {
                                                   TimeExpression aTimeExpression = (TimeExpression) e;
                                                   return theArithmeticExpression(aTimeExpression.getTimeBound());
                                                   }
                                               else
                                                   return "";

                                               }//
                                    }// not OR
                                    }//not And
                                    }// not Equal
                                }
                           }// not a constant
                        }

    }// getBoundAfter

    private static int findStateId (StateMachine aMachine, String theStateName) {
                //System.out.println("Testing:"+theStateName);
                //System.out.println("With initial state name :"+aMachine.getInitialState().getStateName());
                if (aMachine.getInitialState().getStateName().equals(theStateName) ) {
                        return 1; // 0 is the dummy state
                }
                else
                {
                        EList<State> theStates = aMachine.getStates();
                int stateNumber = 2;
                for (Iterator i = theStates.iterator(); i.hasNext(); ) {
                    State aState = (State)  i.next();
                        //System.out.println("With state name :"+aState.getStateName());

                    if (aState.getStateName().equals(theStateName) ) {
                        return stateNumber;
                    }
                    stateNumber++;
                }// for states
                }
                return -1;  // crash, this should not happen
    }// findStateId
            
    private static String  theExpression  (BooleanExpression e) {
        System.err.println(e.getClass().getSimpleName());
        if (e.getClass().getSimpleName().contains("Constant") )
            {   BooleanConstant theConstant = (BooleanConstant) e;
                if (theConstant.isABooleanValue()) {
                                return "true";
                        }
                else
                                return "false";
            }//a constant
        else
            {   if (e.getClass().getSimpleName().contains("Variable") )
                        {
                           BooleanVariableReference aVariableReference = (BooleanVariableReference) e;
                           return aVariableReference.getTheVariable().getVariableID();
                        }
                else
                   { if (e.getClass().getSimpleName().contains("Not") )
                   {  NotExpression theNotExpression = (NotExpression) e;
                           return "(NOT "
                                          + theExpression(  theNotExpression.getTheExpression())
                       + ")";
                   }
                   else {   if (e.getClass().getSimpleName().contains("Equal") )
                                     {  EqualExpression theEqualExpression = (EqualExpression) e;
                                                return "("
                                                                + theArithmeticExpression( theEqualExpression.getLeftExpression() )
                                                                + "=="
                                                                + theArithmeticExpression( theEqualExpression.getRightExpression() )
                                                                +")";
                                      }
                            else { if (e.getClass().getSimpleName().contains("AndExpressionImpl") )
                                {      AndExpression andExpression =(AndExpression) e;
                                return "("
                                        + theExpression( andExpression.getLeftExpression() )
                                        + " AND "
                                        + theExpression( andExpression.getRightExpression() )
                                        +")";

                            }else { if (e.getClass().getSimpleName().contains("OrExpressionImpl") )
                            {  OrExpression orExpression =(OrExpression) e;
                        return "("
                                + theExpression( orExpression.getLeftExpression() )
                                + " OR "
                                + theExpression( orExpression.getRightExpression() )
                                +")";

                            }else {  if (e.getClass().getSimpleName().contains("LessThanExpressionImpl") ) {
                                        LessThanExpression aComparisnon = (LessThanExpression) e;
                                                return "("
                                                                +  theArithmeticExpression(aComparisnon.getLeftExpression())
                                                                + "<"
                                                                + theArithmeticExpression(aComparisnon.getRightExpression())
                                                                +
                                                                ")";
                                       } // not Less Than
                                          else
                                       {  if (e.getClass().getSimpleName().contains("TimeExpressionImpl") ) {
                                           TimeExpression aTimeExpression = (TimeExpression) e;
                                           return "AFTER("+theArithmeticExpression(aTimeExpression.getTimeBound())+")";

                                           }
                                       else
                                           return "";

                                       }//
                            }// not OR
                            }//not And
                            }// not Equal
                        }
                   }// not a constant
                }
    }// theExpression

    private static String  theArithmeticExpression  (ArithmeticExpression e) {
         System.out.println("the arithmetic expresison class: "+e.getClass().getSimpleName());
        if ((e.getClass().getSimpleName().contains("Constant") )
                        &&
                        (! e.getClass().getSimpleName().contains("Predefined") ))
            {  IntegerConstant anIntegerConstant = (IntegerConstant) e;
                   return ""+ anIntegerConstant.getAnIntegerValue()+"";

            }//a constant
        else
            {   if (e.getClass().getSimpleName().contains("Variable") )
                        {
                           IntegerVariableReference aVariableReference = (IntegerVariableReference) e;
                           return aVariableReference.getTheVariable().getVariableID();
                        }
                else if (e.getClass().getSimpleName().contains("Predefined")) {
                          PredefinedIntegerConstant aPredIntConstant = (PredefinedIntegerConstant) e;
                          return aPredIntConstant.getListedIntegerConstant().getIdentifier();

                     } else {  if (e.getClass().getSimpleName().contains("AdditionExpressionImpl")) {
                                    AdditionExpression aSum = (AdditionExpression) e;
                                   return "("
                                                   + theArithmeticExpression(aSum.getLeftExpression())
                                                   +"+"
                                                   + theArithmeticExpression( aSum.getRightExpression())
                                                   +")";
                                 } else
                                 {   if (e.getClass().getSimpleName().contains("SubtractionExpressionImpl")) {
                                         SubtractionExpression theSub =(SubtractionExpression) e;
                                         return "("
                                                                + theArithmeticExpression(theSub.getLeftExpression())
                                                                        +"-"
                                                                + theArithmeticExpression( theSub.getRightExpression())
                                                                +")";

                                         }  else  return "";

                                 }
                     }
                }
    } // theArithmeticExpression

    private String statementsAsString (EList<Statement>  TheStatements) {
        String outputStatementAsString ="";
        for (Iterator t = TheStatements.iterator(); t.hasNext(); ) {
            Statement theStatement = (Statement) t.next();
            System.out.println("Class is :"+ theStatement.getClass().getSimpleName() );
            if (theStatement.getClass().getSimpleName().contains("OutputLine") )
                { OutputLine aStatamentThatPrints = (OutputLine)       theStatement;
                                   outputStatementAsString += "print:"+aStatamentThatPrints.getTheLine()+"\n";
                } // not a print
            else {  if (theStatement.getClass().getSimpleName().contains("AssignmentInteger") ) {
                        AssignmentIntegerLHS theIntegerAssigment = (AssignmentIntegerLHS) theStatement;
                        outputStatementAsString +=theIntegerAssigment.getVariable().getVariableID()+":="
                                          +theArithmeticExpression(theIntegerAssigment.getAnArithmeticExpression())
                                         +"\n";
                        } // not integer assignment
                    else { if (theStatement.getClass().getSimpleName().contains("AssignmentBoolean") ) {
                                AssignmentBooleanLHS theBooleanAssignment = (AssignmentBooleanLHS) theStatement;
                                outputStatementAsString += theBooleanAssignment.getVariable().getVariableID()+":="
                                                                + theExpression(theBooleanAssignment.getABooleanExpression())
                                                                +"\n";
                                 }
                         }
                 }
            }// iterate on statements
        return outputStatementAsString;
    } //statementsAsString

private void buildVisualPanelForMachineWithMachine(VisualPanelForMachine thePanel, StateMachine aMachine, int aNumber ) {
     // the drawing of states and transitions
    String [] stateNamesInVisual = new String[1+aMachine.getStates().size()+1];
            // Lets add the dummy initial state of NuSMV
    stateNamesInVisual[0]="dInit"+aMachine.getName()+" (0) ";
    thePanel.AddState(stateNamesInVisual[0],"");

           // Lets add the initial state
    State initialState = aMachine.getInitialState();
           //System.out.println("Initial State 0 of Machine Name : "+ initialState.getStateName() );
    stateNamesInVisual[1]=initialState.getStateName()+" (1) ";
    if ( (null!= initialState.getOnEntry())
                 && (null!= statementsAsString(initialState.getOnEntry().getStatements() ) ))
                 { //System.out.println("<*>The on entry is :" +  statementsAsString(initialState.getOnEntry().getStatements() )) ;
                       thePanel.AddState(stateNamesInVisual[1], statementsAsString(initialState.getOnEntry().getStatements() ));
                 }
               else
                   thePanel.AddState(stateNamesInVisual[1], "" );

               // The dummy transition
                        thePanel.AddTransition(
                                 stateNamesInVisual[0],     stateNamesInVisual[1],   "1:[true] ("+aNumber+",0)" );
              //comma missing
                //Lets add the remaining states
                EList<State> theStates = aMachine.getStates();
                int stateNumber = 2;
                for (Iterator i = theStates.iterator(); i.hasNext(); ) {
                    State aState = (State)  i.next();
                    //System.out.println("State "+stateNumber+" of Machine Name : "+ aState.getStateName() );
                    stateNamesInVisual[stateNumber] = aState.getStateName()+" ("+stateNumber+")";
                    if ( (null!= aState.getOnEntry()) && (null!=aState.getOnEntry().getStatements() ) )
                    {
                        thePanel.AddState(stateNamesInVisual[stateNumber],
                                 statementsAsString(aState.getOnEntry().getStatements() ));
                    }
                    else
                        thePanel.AddState(stateNamesInVisual[stateNumber],"");
                    stateNumber++;
                }// for states

                //Lets add the transitions
                // Need an array of counters indexed by state to keep track of transition order out of a state
                int [] currentTransitionNumber = new int[stateNamesInVisual.length];
                for (int i=0; i<currentTransitionNumber.length; i++ ) {currentTransitionNumber[i]=1;}
                EList<Transition> theTransitions = aMachine.getTransitions();
                int transitionNumber = 1;
                for (Iterator t = theTransitions.iterator(); t.hasNext(); ) {
                 Transition aTransition = (Transition)  t.next();

                thePanel.AddTransition(
                                 stateNamesInVisual[findStateId(aMachine,aTransition.getSourceState().getStateName())],
                                 stateNamesInVisual[findStateId(aMachine,aTransition.getTargetState().getStateName())],
                                currentTransitionNumber[findStateId(aMachine,aTransition.getSourceState().getStateName())]+":"+
                                 "["+theExpression(aTransition.getBooleanExpression()  )+"]" +" ("+aNumber+","+transitionNumber+")"
                                        //comma missing
                                 );
                currentTransitionNumber[findStateId(aMachine,aTransition.getSourceState().getStateName())]++;
                 transitionNumber++;
                }

            }


private static int  findTotalPredciatedAfter  (EList<StateMachine> theMachines ) {
    int turnNumber = 0;
    int tempTotal = 0;
    for (Iterator m = theMachines.iterator(); m.hasNext(); ) {
                 StateMachine aMachine = (StateMachine)  m.next();
                     tempTotal+= findTotalPredciatedAfter(aMachine);
            }
    return tempTotal;
}

private static int  findTotalPredciatedAfter  (StateMachine aMachine  ) {
    EList<Transition> theTransitions = aMachine.getTransitions();
    int tempTotal = 0;
    int transitionNumber = 1;
    for (Iterator t = theTransitions.iterator(); t.hasNext(); ) {
                 Transition aTransition = (Transition)  t.next();
                tempTotal+=findTotalPredciatedAfter(aTransition);
            }
    return tempTotal;
}

private static int  findTotalPredciatedAfter  (Transition aTransition ) {
    BooleanExpression e = aTransition.getBooleanExpression();
    return findTotalPredciatedAfter(e);
}

private static int  findTotalPredciatedAfter  (ArithmeticExpression e) {
    if (e.getClass().getSimpleName().contains("Constant") )
                                return 0;
    else {  if (e.getClass().getSimpleName().contains("Variable") )
                             return 0;
            else {
                        return 0;
                  }// not a variable
    }// not a constant
}

private static int  findTotalPredciatedAfter  (BooleanExpression e ) {
    if (e.getClass().getSimpleName().contains("Constant") ) return 0;
    else
        {   if (e.getClass().getSimpleName().contains("Variable") ) return 0;
            else
                { if (e.getClass().getSimpleName().contains("NotExpressionImpl") )
                   {  NotExpression theNotExpression = (NotExpression) e;
                           return findTotalPredciatedAfter(theNotExpression.getTheExpression());
                   }
                   else {  if (e.getClass().getSimpleName().contains("Equal") )
                   {  EqualExpression theEqualExpression = (EqualExpression) e;
                      return
                                  findTotalPredciatedAfter(theEqualExpression.getLeftExpression())
                                  +
                                  findTotalPredciatedAfter(theEqualExpression.getRightExpression());
                   }
                   else {  if (e.getClass().getSimpleName().contains("TimeExpressionImpl") )
                   {    TimeExpression tExpression = (TimeExpression) e;
                                return 1+findTotalPredciatedAfter(tExpression.getTimeBound());
                   } else {
                           if ((e.getClass().getSimpleName().contains("AndExpressionImpl") )
                                   || (e.getClass().getSimpleName().contains("OrExpressionImpl") ) ) {
                                      if (e.getClass().getSimpleName().contains("AndExpressionImpl") )
                                                                {
                                                                        AndExpression andExpression =(AndExpression) e;
                                                                        return findTotalPredciatedAfter(andExpression.getLeftExpression())
                                                                                        + findTotalPredciatedAfter(andExpression.getRightExpression());
                                                                }
                                                                else {
                                                                        OrExpression orExpression =(OrExpression) e;
                                                                        return findTotalPredciatedAfter(orExpression.getLeftExpression())
                                                                                        + findTotalPredciatedAfter(orExpression.getRightExpression());

                                                                }

                           }else {
                                   if (e.getClass().getSimpleName().contains("Less") )
                                   { LessThanExpression lthExpression = (LessThanExpression) e;
                                     return findTotalPredciatedAfter(lthExpression.getLeftExpression())+
                                                 findTotalPredciatedAfter(lthExpression.getRightExpression());

                                   } else {
                                           return 0;
                                   }// not Lees than

                           }// not and or Or

                   } // not a Time ie AFter predicate
                   }// not an equality test

                   }// not a negation

                   }// not variable
        } // not constant
} //end findTotalPredciatedAfter

    private void cleanAllVariables() {
        if (null!=JTextFieldBooleanSensorVariablesValues) {
                for (int i =0;
                            i< JTextFieldBooleanSensorVariablesValues.length; i++) {
                        JTextFieldBooleanSensorVariablesValues[i].setBackground(Color.WHITE);
                }
        }// have Boolean Sensor Variables
        if (null!=JTextFieldIntegerSensorVariablesValues) {
                        for (int i =0;
                                    i< JTextFieldIntegerSensorVariablesValues.length; i++) {
                                JTextFieldIntegerSensorVariablesValues[i].setBackground(Color.WHITE);
                        }
        }// have Integer Sensor Variables

        if (null!=JTextFieldBooleanExternalVariablesValues) {
                        for (int i =0;
                                    i< JTextFieldBooleanExternalVariablesValues.length; i++) {
                                JTextFieldBooleanExternalVariablesValues[i].setBackground(Color.WHITE);
                        }
                        }// have Boolean External Variables
                   if (null!=JTextFieldIntegerExternalVariablesValues) {
                                for (int i =0;
                                            i< JTextFieldIntegerExternalVariablesValues.length; i++) {
                                        //System.out.println("index"+i);
                                        JTextFieldIntegerExternalVariablesValues[i].setBackground(Color.WHITE);
                                }
                                }// have Integer External Variables

                        if (null!=JTextFieldBooleanWhiteboardVariablesValues) {
                        for (int i =0;
                                    i< JTextFieldBooleanWhiteboardVariablesValues.length; i++) {
                                JTextFieldBooleanWhiteboardVariablesValues[i].setBackground(Color.WHITE);
                        }
                        }// have Boolean White-board Variables

                        if (null!=JTextFieldIntegerWhiteboardVariablesValues) {
                                for (int i =0;
                                            i< JTextFieldIntegerWhiteboardVariablesValues.length; i++) {
                                        JTextFieldIntegerWhiteboardVariablesValues[i].setBackground(Color.WHITE);
                                }
                                }// have Integer White-board Variables

                        if (null!=JTextFieldIntegerLocalVariablesValues) {
                                for (int i =0;
                                            i< JTextFieldIntegerLocalVariablesValues.length; i++) {
                                        // get the array list
                                        if (null!=JTextFieldIntegerLocalVariablesValues[i]) {
                                        Iterator it = JTextFieldIntegerLocalVariablesValues[i].iterator();
                                        while (it.hasNext() ) {
                                                javax.swing.JTextField theTextFiled = (javax.swing.JTextField) it.next();
                                                theTextFiled.setBackground(Color.WHITE);
                                        }
                                        }
                                }
                                }// have Integer Local Variables

                           if (null!=JTextFieldBooleanLocalVariablesValues) {
                                        for (int i =0;
                                                    i< JTextFieldBooleanLocalVariablesValues.length; i++) {
                                                if (null!=JTextFieldBooleanLocalVariablesValues[i] ) {
                                                Iterator it = JTextFieldBooleanLocalVariablesValues[i].iterator();
                                                while (it.hasNext() ) {
                                                        javax.swing.JTextField theTextFiled = (javax.swing.JTextField) it.next();
                                                        theTextFiled.setBackground(Color.WHITE);
                                                }}

                                        }
                                        }// have Boolean Local Variables
    }
    
    private int machineNumberFromMachineName(String theName) {
                EList<StateMachine> theMachines = theArrangemant.getStateMachine();
                 int result = -1;
                if (null != theMachines) {
                        int i =0;
                        for (Iterator m = theMachines.iterator(); m.hasNext(); ) {
                                StateMachine aMachine = (StateMachine)  m.next();
                                     if ( aMachine.getName().contentEquals(theName) ) {result =i; }
                                i++;
                         }// iterate on machines
                 }// we have machines


                return result;
    } //machineNumberFromMachineName
    
	    private void UpdateValueGUI(String theMachineName, String theValue , String theVariable) {
	    	// search for the variable in the global ones
	    	
	    	if (null!=jLabelBooleanSensorVariablesNames) {
		    	for (int i =0; 
		    	            i< jLabelBooleanSensorVariablesNames.length; i++) {
		    		if(jLabelBooleanSensorVariablesNames[i].getText().contains(theVariable))
		    		{
		    			JTextFieldBooleanSensorVariablesValues[i].setBackground(Color.YELLOW);
		    			JTextFieldBooleanSensorVariablesValues[i].setText(theValue);
		    			return; // exit if found
		    		}
		    	}
	         }// there are Boolean Sensor Variables
	    	
	    	if (null!=jLabelIntegerSensorVariablesNames) {
		    	for (int i =0; 
		    	            i< jLabelIntegerSensorVariablesNames.length; i++) {
		    		if(jLabelIntegerSensorVariablesNames[i].getText().contains(theVariable))
		    		{
		    			JTextFieldIntegerSensorVariablesValues[i].setBackground(Color.YELLOW);
		    			JTextFieldIntegerSensorVariablesValues[i].setText(theValue);
		    			return; // exit if found
		    		}
		    	}
	         }// there are Integer Sensor Variables
	    	
	    	if (null!=jLabelBooleanExternalVariablesNames) {
		    	for (int i =0; 
		    	            i< jLabelBooleanExternalVariablesNames.length; i++) {
		    		if(jLabelBooleanExternalVariablesNames[i].getText().contains(theVariable))
		    		{
		    			JTextFieldBooleanExternalVariablesValues[i].setBackground(Color.YELLOW);
		    			JTextFieldBooleanExternalVariablesValues[i].setText(theValue);
		    			return; // exit if found
		    		}
		    	}
	         }// there are Boolean External Variables
	    	
	    	if (null!=jLabelIntegerExternalVariablesNames) {
		    	for (int i =0; 
		    	            i< jLabelIntegerExternalVariablesNames.length; i++) {
		    		if(jLabelIntegerExternalVariablesNames[i].getText().contains(theVariable))
		    		{
		    			JTextFieldIntegerExternalVariablesValues[i].setBackground(Color.YELLOW);
		    			JTextFieldIntegerExternalVariablesValues[i].setText(theValue);
		    			return; // exit if found
		    		}
		    	}
	         }// there are Integer External Variables
	    	
	    	if (null!=jLabelBooleanWhiteboardVariablesNames) {
		    	for (int i =0; 
		    	            i< jLabelBooleanWhiteboardVariablesNames.length; i++) {
		    		if(jLabelBooleanWhiteboardVariablesNames[i].getText().contains(theVariable))
		    		{

		    			JTextFieldBooleanWhiteboardVariablesValues[i].setBackground(Color.YELLOW);
		    			JTextFieldBooleanWhiteboardVariablesValues[i].setText(theValue);
		    			return; // exit if found
		    		}
		    	}
	         }// there are Boolean White-board Variables
	    	
	    	if (null!=jLabelIntegerWhiteboardVariablesNames) {
		    	for (int i =0; 
		    	            i< jLabelIntegerWhiteboardVariablesNames.length; i++) {
		    		if(jLabelIntegerWhiteboardVariablesNames[i].getText().contains(theVariable))
		    		{

		    			JTextFieldIntegerWhiteboardVariablesValues[i].setBackground(Color.YELLOW);
		    			JTextFieldIntegerWhiteboardVariablesValues[i].setText(theValue);
		    			return; // exit if found
		    		}
		    	}
	         }// there are Integer White-board Variables
	    	
	    	// Now the specific machine variables are updated
	    	if (null!=JTextFieldIntegerLocalVariablesValues) {
	    		System.out.println("Seeking machine named "+theMachineName);
	    		int numericalForTheMachine = machineNumberFromMachineName(theMachineName);
	    		if ( ( numericalForTheMachine<0) && (totalPredicatesAfter>0) ) // this name may be a timer
	    		{ // need to recognize which timer this belongs to
	    			for (int timerID=0; timerID< totalPredicatesAfter; timerID++) {
	    				int netIndex = theArrangemant.getStateMachine().size()+timerID;
	    				if (jLabelMachineNames[netIndex].getText().contains(theMachineName)) {
	    					System.out.println("Found timmer named "+theMachineName);
	    					if (null!=JTextFieldIntegerLocalVariablesValues[netIndex]) {
	    			    		Iterator it = JTextFieldIntegerLocalVariablesValues[netIndex].iterator();
	    			    		Iterator itVar = jLabelIntegerLocalVariablesNames[netIndex].iterator();
	    			    		while (it.hasNext() && itVar.hasNext() ) {
	    			    			javax.swing.JTextField theTextFiled = (javax.swing.JTextField) it.next();
	    			    			javax.swing.JLabel theVariableLabel = (javax.swing.JLabel) itVar.next();
	    			    			if  ( theVariableLabel.getText().contains(theVariable) ) {
	    			    				theTextFiled.setBackground(Color.YELLOW);
	    			    				theTextFiled.setText(theValue);
	    			    			}
	    			    		  } // while
	    			    	    }// if
	    				}//if
	    			}//for
	    		} else {
		    		// get the array list
		    		if (null!=JTextFieldIntegerLocalVariablesValues[numericalForTheMachine]) {
		    		Iterator it = JTextFieldIntegerLocalVariablesValues[numericalForTheMachine].iterator();
		    		Iterator itVar = jLabelIntegerLocalVariablesNames[numericalForTheMachine].iterator();
		    		while (it.hasNext() && itVar.hasNext() ) {
		    			javax.swing.JTextField theTextFiled = (javax.swing.JTextField) it.next();
		    			javax.swing.JLabel theVariableLabel = (javax.swing.JLabel) itVar.next();
		    			if  ( theVariableLabel.getText().contains(theVariable) ) {
		    				theTextFiled.setBackground(Color.YELLOW);
		    				theTextFiled.setText(theValue);
		    			}
		    		  } // while
		    	    }// if
	    		} // not a timer
		    	}// have Integer Local Variables
	    	
	    	if (null!=JTextFieldBooleanLocalVariablesValues) {
	    		int numericalForTheMachine = machineNumberFromMachineName(theMachineName);
	    		if ( ( numericalForTheMachine<0) && (totalPredicatesAfter>0) ) // this name may be a timer
	    		{
	    			for (int timerID=0; timerID< totalPredicatesAfter; timerID++) {
	    				int netIndex = theArrangemant.getStateMachine().size()+timerID;
	    				if (jLabelMachineNames[netIndex].getText().contains(theMachineName)) {
	    					System.out.println("Found timmer named "+theMachineName);
	    					if (null!=JTextFieldBooleanLocalVariablesValues[netIndex]) {
	    			    		Iterator it = JTextFieldBooleanLocalVariablesValues[netIndex].iterator();
	    			    		Iterator itVar = jLabelBooleanLocalVariablesNames[netIndex].iterator();
	    			    		while (it.hasNext() && itVar.hasNext() ) {
	    			    			javax.swing.JTextField theTextFiled = (javax.swing.JTextField) it.next();
	    			    			javax.swing.JLabel theVariableLabel = (javax.swing.JLabel) itVar.next();
	    			    			if  ( theVariableLabel.getText().contains(theVariable) ) {
	    			    				theTextFiled.setBackground(Color.YELLOW);
	    			    				theTextFiled.setText(theValue);
	    			    			}//if
	    			    		}//while
	    			    	 }//if
	    				}//if
	    			}//for
	    		} else {
		    		// get the array list
		    		if (null!=JTextFieldBooleanLocalVariablesValues[numericalForTheMachine]) {
		    		Iterator it = JTextFieldBooleanLocalVariablesValues[numericalForTheMachine].iterator();
		    		Iterator itVar = jLabelBooleanLocalVariablesNames[numericalForTheMachine].iterator();
		    		while (it.hasNext() && itVar.hasNext() ) {
		    			javax.swing.JTextField theTextFiled = (javax.swing.JTextField) it.next();
		    			javax.swing.JLabel theVariableLabel = (javax.swing.JLabel) itVar.next();
		    			if  ( theVariableLabel.getText().contains(theVariable) ) {
		    				theTextFiled.setBackground(Color.YELLOW);
		    				theTextFiled.setText(theValue);
		    			}//if
		    		}//while
		    	 }//if
	    		}// else
		    	}// have Boolean Local Variables
	    
	    }  /* UpdateValueGUI */


    /**
     * 
     * @return the next line so we can try to do something clever next time
     * ERROR is returned hoping no nusmv trace has this as a single line
     */
    private String readNextKripkeStateAndUpdateGUI ()
        {   cleanAllVariables();
	    	while (scanner.hasNextLine()) {
	    		String theInputLine = scanner.nextLine();
	    		if (theInputLine.contains("  -> State:"))
	    		{    String result[] =  theInputLine.split(":");
   			         String noBlanks[] = result[1].split("\\s");
   			         System.out.println("Kripke State Found"+ noBlanks[1]);
					 return noBlanks[1];
	    		}
	    		else {
	    			if (theInputLine.contains("turn")) {
		    			 String result[] =  theInputLine.split("=");
		    			 //System.out.println("Machine ID:"+result[1]);
		    			 for (int  machineId=0; machineId<jLabelMachineNames.length; machineId++)
		    			 {
		    				 if (machineId==Integer.parseInt(result[1].trim())  ) {
		    					 jTextFieldMachineTurnNumber[machineId].setBackground(Color.YELLOW);
		    					 theCurrentTurn = machineId;
		    					 //System.out.println("Reseting Machine ID:"+machineId);
		    				 } // set background to yellow of machine in turn
		    				 else
		    				 {
		    					 jTextFieldMachineTurnNumber[machineId].setBackground(goodFieldColor);
		    				 }
		    			 }
	    			} // a turn
	    			else {
	    				if (theInputLine.contains(".pc")) {
						       String possibleMachine[] =  theInputLine.split("\\.");
						       String theMachineName = possibleMachine[0].trim();
						       //System.out.println("Suspect a Program counter:"+theMachineName);
						       int numericalForTheMachine = machineNumberFromMachineName(theMachineName);
						       //System.out.println("Highligthing"+numericalForTheMachine);
						       String possibleStateNumber[] =  theInputLine.split("=");
						       String thePCofState = possibleStateNumber[1].trim();
						       //System.out.println("Suspect value of pc counter:"+thePCofState);
						       int numericalPCofState = Integer.parseInt(thePCofState);
						       if ((0<=numericalForTheMachine ) &&
						         (numericalForTheMachine< jPanelVisualMachines.length))
						       {
						    	   jPanelVisualMachines[numericalForTheMachine].SetPCState(thePCofState);
						       } else { // we may need to update a timer
						    	   for (int timerId=0; timerId<totalPredicatesAfter;timerId++ ) {
							    		  int newIndex = theArrangemant.getStateMachine().size()+timerId;
							    		  if (jLabelMachineNames[newIndex].getText().contains(theMachineName) )
							    		  {
							    			  jPanelVisualMachines[newIndex].SetPCState(thePCofState);
							    		  }
							    	  }// for
						       }
					       } // not a pc line
	    				  else { String prexisForTransition = "condT";
	    					  // Highlight the Transition that would fire next
	    					  if (theInputLine.contains("."+prexisForTransition)) {
	    						  String possibleMachineAndTransition[] =  theInputLine.split("\\.");
	    						  String theMachineName = possibleMachineAndTransition[0].trim();
							      // System.out.println("Suspect machine for a transition:"+theMachineName);
	    						  String theMachineNumberAndTransitionNumber = possibleMachineAndTransition[1].trim();
							      //System.out.println("Suspect for a transitionID:"+theMachineNumberAndTransitionNumber);
							      String plainMachineNumberAndTransiitonNumber = theMachineNumberAndTransitionNumber.substring(prexisForTransition.length());
							      int numericalForTheMachine = machineNumberFromMachineName(theMachineName);
							      //String machineNumberAsString =""+numericalForTheMachine;
							      
							      
							      String possibleTransitionValue[] =  plainMachineNumberAndTransiitonNumber.split("=");
							      //System.out.println("Suspect for a transition value:"+possibleTransitionValue[1]);
							      
							      if  ( (possibleTransitionValue[1].contains("TRUE")) 
							    	  && (theCurrentTurn==numericalForTheMachine) ) // transition firing in machine with turn
							        { // highlight the transition
							    	  //System.out.println("Highlighting TRANSITION VALUE:"+"("+possibleTransitionValue[0].trim()+") with turn:"+theCurrentTurn );
							    	  // need to add a comma that separates state-machine ID and TransitionID
							    	  String machineID=""+theCurrentTurn;
							    	  String transitionID=possibleTransitionValue[0].trim().substring(machineID.length());
							    	  //System.out.println("TRANSITION Search Key:"+"("+  "("+machineID+","+transitionID+")" );
							    	  jPanelVisualMachines[numericalForTheMachine].highlightTransition("("+machineID+","+transitionID+")" );
							      }		else { // maybe it is a timer
							    	  for (int timerId=0; timerId<totalPredicatesAfter;timerId++ ) {
							    		  int newIndex = theArrangemant.getStateMachine().size()+timerId;
							    		  if (jLabelMachineNames[newIndex].getText().contains(theMachineName) )
							    		  {
							    			  if  ( (possibleTransitionValue[1].contains("TRUE")) 
											    	  && (theCurrentTurn==newIndex) ) {
							    				  String machineID=""+theCurrentTurn;
										    	  String transitionID=possibleTransitionValue[0].trim().substring(machineID.length());
										    	  jPanelVisualMachines[newIndex].highlightTransition("("+machineID+","+transitionID+")" );
							    				  
							    			  }// if test firing
							    		  }
							    	  }// for
							    	  
							      }//maybe a timer
	    					  } // not a machine with a firing transition
	    					  else {
	    						  String prexisForNonTransition = ".condDefault";
	    						  if (theInputLine.contains(prexisForNonTransition)) {
	    							  String possibleMachineName[] =  theInputLine.split("\\.");
	    							  String theMachineName = possibleMachineName[0].trim();
	    							  //System.out.println("**> theInputLine:"+theInputLine);
								      //System.out.println("Suspect machine for a NON transition:"+theMachineName);
								      int numericalForTheMachine = machineNumberFromMachineName(theMachineName);
								      // Maybe a timer
								      if ((numericalForTheMachine<0) && (totalPredicatesAfter>0)) {
								    	  for (int timerId=0; timerId<totalPredicatesAfter;timerId++ ) {
								    		  int newIndex = theArrangemant.getStateMachine().size()+timerId;
								    		  if (jLabelMachineNames[newIndex].getText().contains(theMachineName) )
								    		  {
								    			  jPanelVisualMachines[newIndex].regularTransitionColor();
								    		  }
								    	  }// for
								      } else
								       {jPanelVisualMachines[numericalForTheMachine].regularTransitionColor();}
	    							  
	    						  } // do not highlight transitions
	    						  else {  
	    							  if ((theInputLine.contains(".")) && (theInputLine.contains("=")) ) {
	    								  // an assignment in a Module
	    								  String possibleMachineName[] =  theInputLine.split("\\.");
		    							  String theMachineName = possibleMachineName[0].trim();
									      //System.out.println("Suspect machine for an assignment:"+theMachineName);
									      String possibleValue[] =  theInputLine.split("=");
									      //System.out.println("Suspect value:"+possibleValue[1]);
									      String possibleVariable[] = possibleMachineName[1].split("=");
									      //System.out.println("Suspect variable:"+possibleVariable[0]);
									      
	    							      UpdateValueGUI(theMachineName,possibleValue[1],possibleVariable[0].trim());
	    						         } else // maybe a variable of main
	    						         {
	    						        	 if (theInputLine.contains("=")) {
	    						        		 String possibleGlobalVariableAndValue[] =  theInputLine.split("=");
	    						        		 String theVariable = possibleGlobalVariableAndValue[0].trim();
	    						        		 String theValue = possibleGlobalVariableAndValue[1].trim();
	    						        		System.out.println("Suspected global variable:"+theVariable+" with value:"+theValue);
	    						        		 // search in the GUI the global variables
	    						        		// Integer Sensor Variables
	    						        		if (null!=jLabelIntegerSensorVariablesNames) {
	    						        		 for (int index=0; index< jLabelIntegerSensorVariablesNames.length; index++ ) {
	    						        			 System.out.println("Inspecting:"+jLabelIntegerSensorVariablesNames[index].getText());
	    						        			 if ( jLabelIntegerSensorVariablesNames[index].getText().contains(theVariable) ) {
	    						        				 // update with value
	    						        				 JTextFieldIntegerSensorVariablesValues[index].setBackground(Color.YELLOW);
	    						        				 JTextFieldIntegerSensorVariablesValues[index].setText(theValue);
	    						        			 }
	    						        		 }}
	    						        		// Boolean Sensor Variables
	    						        		if (null!=jLabelBooleanSensorVariablesNames) {
	    						        		 for (int index=0; index< jLabelBooleanSensorVariablesNames.length; index++ ) {
	    						        			 System.out.println("Inspecting:"+jLabelBooleanSensorVariablesNames[index].getText());
	    						        			 if ( jLabelBooleanSensorVariablesNames[index].getText().contains(theVariable) ) {
	    						        				 // update with value
	    						        				 JTextFieldBooleanSensorVariablesValues[index].setBackground(Color.YELLOW);
	    						        				 JTextFieldBooleanSensorVariablesValues[index].setText(theValue);
	    						        			 }
	    						        		 }}
	    						           		// Integer Effector Variables
	    						        		if (null!=jLabelIntegerExternalVariablesNames) {
	    						        		 for (int index=0; index< jLabelIntegerExternalVariablesNames.length; index++ ) {
	    						        			 System.out.println("Inspecting:"+jLabelIntegerExternalVariablesNames[index].getText());
	    						        			 if ( jLabelIntegerExternalVariablesNames[index].getText().contains(theVariable) ) {
	    						        				 // update with value
	    						        				 JTextFieldIntegerExternalVariablesValues[index].setBackground(Color.YELLOW);
	    						        				 JTextFieldIntegerExternalVariablesValues[index].setText(theValue);
	    						        			 }
	    						        		 }}
	    						        		// Boolean Effector Variables
	    						        		if (null!=jLabelBooleanExternalVariablesNames) {
	    						        		 for (int index=0; index< jLabelBooleanExternalVariablesNames.length; index++ ) {
	    						        			 System.out.println("Inspecting:"+jLabelBooleanExternalVariablesNames[index].getText());
	    						        			 if ( jLabelBooleanExternalVariablesNames[index].getText().contains(theVariable) ) {
	    						        				 // update with value
	    						        				 JTextFieldBooleanExternalVariablesValues[index].setBackground(Color.YELLOW);
	    						        				 JTextFieldBooleanExternalVariablesValues[index].setText(theValue);
	    						        			 }
	    						        		 }}
	    							           		// Integer whiteboard Variables
	    						        		if (null!=jLabelIntegerWhiteboardVariablesNames) {
	    						        		 for (int index=0; index< jLabelIntegerWhiteboardVariablesNames.length; index++ ) {
	    						        			 System.out.println("Inspecting:"+jLabelIntegerWhiteboardVariablesNames[index].getText());
	    						        			 if ( jLabelIntegerWhiteboardVariablesNames[index].getText().contains(theVariable) ) {
	    						        				 // update with value
	    						        				 JTextFieldIntegerWhiteboardVariablesValues[index].setBackground(Color.YELLOW);
	    						        				 JTextFieldIntegerWhiteboardVariablesValues[index].setText(theValue);
	    						        			 }
	    						        		 }}
	    						        		// Boolean whiteboard Variables
	    						        		if (null!=jLabelBooleanWhiteboardVariablesNames) {
	    						        		 for (int index=0; index< jLabelBooleanWhiteboardVariablesNames.length; index++ ) {
	    						        			 System.out.println("Inspecting:"+jLabelBooleanWhiteboardVariablesNames[index].getText());
	    						        			 if ( jLabelBooleanWhiteboardVariablesNames[index].getText().contains(theVariable) ) {
	    						        				 // update with value
	    						        				 JTextFieldBooleanWhiteboardVariablesValues[index].setBackground(Color.YELLOW);
	    						        				 JTextFieldBooleanWhiteboardVariablesValues[index].setText(theValue);
	    						        			 }
	    						        		 }}
	    						        		 
	    						        	 } else
	    		    						        System.out.println("Not an interpreted line line:"+theInputLine);

	    						         }
	    						  } // not an assignment in a module
	    					  }
						       
						        //return "Something";
					       }
	    				
	    			}
	    			
	    		}// not a state line
	    		
	    	}// read lines
	    	return null; // signal and end of file
} // readNextKripkeStateAndUpdateGUI



/**
***************************************
*  The Handlers
*/
    private void jMenuItemExitActionPerformed(java.awt.event.ActionEvent evt) {
                     System.exit(0);
    } /* jMenuItemExitActionPerformed */
    
 
                 private void jMenuExpandViewLLFSMActionPerformed(java.awt.event.ActionEvent  evt) {
                         // provide opportunity to chose an LLFSM and to show it larger
                         EList<StateMachine> theMachines = theArrangemant.getStateMachine();
                         Object[] possibilities = new Object[theMachines.size()+totalPredicatesAfter];
                         if (null != theMachines) { int i=0;
                                         for (Iterator m = theMachines.iterator(); m.hasNext(); ) {
                                                 StateMachine aMachine = (StateMachine)  m.next();
                                                 possibilities[i]=i+" "+aMachine.getName();
                                                 i++;
                                           }// iterate
                                         //TEST if we have timers
                                         if (totalPredicatesAfter>0) {
                                                 for (int timerId=0; timerId<totalPredicatesAfter;timerId++ ) {
                                                         int newIndex = theMachines.size()+timerId;
                                                         possibilities[newIndex]=newIndex+" "+jLabelMachineNames[newIndex].getText();
                                                         
                                                 }// insert possibilities
                                         }// do we have timers
                                         }
                         
                         String s = (String)JOptionPane.showInputDialog(
                                             this,
                                             "Chose a LLFSM to expand its view",
                                             "Select an LLFSM in the arrangement",
                                             JOptionPane.PLAIN_MESSAGE,
                                             UIManager.getIcon("FileChooser.listViewIcon"),
                                             possibilities,
                                             possibilities[0]);
                         
                         //If a string was returned, say so.
                         if ((s != null) && (s.length() > 0)) {
                                 javax.swing.JFrame popUpFrame = new javax.swing.JFrame();
                                 popUpFrame.setTitle("Visulaising Machine "+s);
                                 popUpFrame.setDefaultCloseOperation(javax.swing.JFrame.EXIT_ON_CLOSE);
                                 
                                 VisualPanelForMachine toShowSelectedMachine = new VisualPanelForMachine();
                                 String result[] = s.split("\\s");
                                 int indexForMachine = Integer.parseInt(result[0]);
                                  if ((0<= indexForMachine) && (indexForMachine< theMachines.size()))
                                  {  
                                     StateMachine aMachine = theMachines.get(indexForMachine);
                                     buildVisualPanelForMachineWithMachine(toShowSelectedMachine,aMachine,indexForMachine);
                                     toShowSelectedMachine.AutoLayout();
                                     popUpFrame.getContentPane().add(toShowSelectedMachine, BorderLayout.CENTER);
                                     popUpFrame.pack();
                                     popUpFrame.setVisible(true);
                                 return;
                                  }
                                  else { // draw a timer
                                          if ( indexForMachine>= theMachines.size() ) {
                                                  int timerIndex = indexForMachine - theMachines.size();
                                                  // the bound is set to 3 should be set properly
                                                  String theParts[] = jLabelMachineNames[indexForMachine].getText().split("_");
                                                  buildVisualPanelForTimer(toShowSelectedMachine,
                                                                  theParts[0]+"_"+theParts[1],
                                                                  theParts[2], 
                                                                  theParts[3], 
                                                                  Integer.parseInt(theParts[4].trim()), 3, 1, indexForMachine);
                                                 toShowSelectedMachine.AutoLayout();
                                             popUpFrame.getContentPane().add(toShowSelectedMachine, BorderLayout.CENTER);
                                             popUpFrame.pack();
                                             
                                             popUpFrame.setVisible(true);
                                         return;
                                          }// test index of timer
                                  } // when drawing a timer
                         }
                         //If you're here, the return value was null/empty.
                         //setLabel("Come on, finish the sentence!");
     } /* jMenuExpandViewLLFSMActionPerformed */    
    
    private void jMenuItemNextActionPerformed(java.awt.event.ActionEvent  evt) {
                         //Pattern p = Pattern.compile(".* -> State:.*<-.*");
                        //Pattern p = Pattern.compile("State");
                         //Pattern p = Pattern.compile("\\s"); // State:.*");

        if (null!=IDofKripkeStateAboutToBeRead) {
                                         jTextFieldKripkeState.setText(IDofKripkeStateAboutToBeRead);
                                         jTextFieldKripkeState.setBackground(Color.YELLOW);
                                         IDofKripkeStateAboutToBeRead = readNextKripkeStateAndUpdateGUI();
                         }
        else {  scanner.close();
                                // need to present warning
                JOptionPane.showMessageDialog(this, "End of NuSMV trace.",  "New NUSMV tarce needed", JOptionPane.WARNING_MESSAGE);

                                // we cannot read the trace any more
                jButtonNext.setEnabled(false);
                jMenuItemNext.setEnabled(false);
                                // maybe open a new NuSMV trace
                jMenuItemOpenNuSSMV.setEnabled(true);

              }
    } //jMenuItemNextActionPerformed

    private void jMenuItemOpenNuSMVActionPerformed(java.awt.event.ActionEvent  evt) {
                         OpenMachineFileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
                     OpenMachineFileChooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
                     boolean userNavigating=false;
                     do {   int returnVal = OpenMachineFileChooser.showOpenDialog(this);
                        if (returnVal == JFileChooser.APPROVE_OPTION) {
                                File file = OpenMachineFileChooser.getSelectedFile();
                                if (file.getName().contains(Text.NUSMV_TRACE_EXTENSION)) {
                                        userNavigating=false; // no longer searching
                                        try {
                                                                scanner = new Scanner(file);
                                                                // we consume the header of the trace until we find a state
                                                                boolean LineIdentifyingStateFound = false;
                                                                while (  (! LineIdentifyingStateFound) && (scanner.hasNextLine() )) {
                                                                        String theInput = scanner.nextLine();
                                                                         if (theInput.contains("  -> State:")) {
                                                                                 String result[] =  theInput.split(":");
                                                                         String noBlanks[] = result[1].split("\\s");
                                                                         System.out.println("State Found"+ noBlanks[1]);
                                                                         // sore string found in global string
                                                                         IDofKripkeStateAboutToBeRead = noBlanks[1];
                                                                         LineIdentifyingStateFound = true;
                                                                         }// testing a line
                                                                }
                                                                if (!LineIdentifyingStateFound) { // bad input nusmv file
                                                                        JOptionPane.showMessageDialog(this, "Faulty NuSMV trace.",  "Can Not Finf Kripke State", JOptionPane.WARNING_MESSAGE);
                                                System.exit(0); // Maybe something more friendly could be done
                                            }
                                                        } catch (FileNotFoundException e) {
                                                                e.printStackTrace();
                                                        }
                                        // file is open enable NEXT
                                        jTextFieldTraceFile.setText(file.getName());
                                                jMenuItemOpenNuSSMV.setEnabled(false);
                                                // if we have a model (arrangement), we should be able to emulate the trace
                                                if (! jMenuItemOpen.isEnabled() ) {
                                                        jButtonNext.setEnabled(true);
                                                        jMenuItemNext.setEnabled(true);
                                                }



                                }// the file has nusmv trace extension
                                else
                                { userNavigating=true; // reset the file chooser
                                  OpenMachineFileChooser.setCurrentDirectory(file);
                                }

                        }// the user approved

                     } while (userNavigating);

    }// end handler jMenuItemOpenNuSMVActionPerformed for opening NUSMV file
    
    /**
     * attempt to redraw the machines better
     * @param evt 
     */
    private void jMenuItemRedrawActionPerformed(java.awt.event.ActionEvent  evt) {
	EList<StateMachine> theMachines = theArrangemant.getStateMachine();
	if (null != theMachines) { 
	    int i=0;
	    for (Iterator m = theMachines.iterator(); m.hasNext(); ) {
                    StateMachine aMachine = (StateMachine)  m.next();
                    EList<State> theStates = aMachine.getStates();
                    int xDimension = 250+5*theStates.size();
                    int yDimension = 300+10*theStates.size();
                        	 //System.err.println("Machine"+aMachine.getName());
                    if (jLabelIntegerLocalVariablesPanel[i].getWidth()>xDimension)
                        	 { //System.err.println("Changing x dimension Integer larger than"+xDimension);
                        	   xDimension = jLabelIntegerLocalVariablesPanel[i].getWidth();
                        	 }
                    if (jLabelBooleanLocalVariablesPanel[i].getWidth()>xDimension)
                           	 {  //System.err.println("Changing x dimension Boolean larger than"+xDimension);
                           	 xDimension = jLabelBooleanLocalVariablesPanel[i].getWidth();
                           	 }
                    if (jPanelInforMachinesArea.getWidth()>xDimension)
                        	 {  //System.err.println("Changing x dimension Boolean larger than"+xDimension);
                               	 xDimension = jPanelInforMachinesArea.getWidth();
                             }
                    if (jPanelInforMachinesArea.getHeight()>yDimension)
                        	 {  //System.err.println("Changing x dimension Boolean larger than"+xDimension);
                        		 yDimension = jPanelInforMachinesArea.getHeight();
                                 }
                        
                    jPanelVisualMachines[i].setPreferredSize(new  Dimension(xDimension, yDimension ));
                    jPanelVisualMachines[i].revalidate();
                        	 // also the variables
                        	 /*
                        	 jLabelIntegerLocalVariablesPanel[i].setPreferredSize(new
                        			 Dimension(xDimension,30));
                        	 jLabelIntegerLocalVariablesPanel[i].revalidate();
                        	 */
                    i++;                        	 
	    } // iterate over a machine
	        			 
	     if (totalPredicatesAfter>0)  { // we have timers 
	        for (int timerId=0; timerId < totalPredicatesAfter; timerId++) {
	        	int newIndex =  theMachines.size()+ timerId;
	        	int xDimension = 250+5*5;
	                int yDimension = 300+10*5;
	                if (jLabelIntegerLocalVariablesPanel[newIndex].getWidth()>xDimension)
	                    { //System.err.println("Changing x dimension Integer larger than"+xDimension);
	                        	   xDimension = jLabelIntegerLocalVariablesPanel[newIndex].getWidth();
	                    }
	                if (jLabelBooleanLocalVariablesPanel[newIndex].getWidth()>xDimension)
	                    {  //System.err.println("Changing x dimension Boolean larger than"+xDimension);
	                           	 xDimension = jLabelBooleanLocalVariablesPanel[newIndex].getWidth();
	                     }
	                if (jPanelInforMachinesArea.getWidth()>xDimension)
	                    {  //System.err.println("Changing x dimension Boolean larger than"+xDimension);
	                               	 xDimension = jPanelInforMachinesArea.getWidth();
	                    }
	                if (jPanelInforMachinesArea.getHeight()>yDimension)
	                    {  //System.err.println("Changing x dimension Boolean larger than"+xDimension);
	                        		 yDimension = jPanelInforMachinesArea.getHeight();
	                     }
	                        
	                 jPanelVisualMachines[newIndex].setPreferredSize(new Dimension(xDimension, yDimension ));
	                 jPanelVisualMachines[newIndex].AutoLayout();
	                 jPanelVisualMachines[newIndex].revalidate();			 
	        }
	     }// we have timers       			 
	}
    } // jMenuItemRedrawActionPerformed

private void jMenuItemOpenActionPerformed(java.awt.event.ActionEvent evt) {                                              
	             OpenMachineFileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
	             OpenMachineFileChooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
	             boolean userNavigating=false;
	             
	             do {   int returnVal = OpenMachineFileChooser.showOpenDialog(this);
	                if (returnVal == JFileChooser.APPROVE_OPTION) {
	                File file = OpenMachineFileChooser.getSelectedFile();
	                if (file.getName().contains(Text.ARRANGEMENT_MODEL_EXTENSION))
	                { userNavigating=false;
	                  //try {  
	                         newGITmetamodelwithtypes.NewGITmetamodelwithtypesPackage aPackage = newGITmetamodelwithtypes.NewGITmetamodelwithtypesPackage.eINSTANCE;
	                         
	                         ResourceSet resourceSet = new ResourceSetImpl();
	                         resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().
	     		        put(Text.ARRANGEMENT_MODEL_EXTENSION, new XMIResourceFactoryImpl());
	                         
	                         //System.out.println("File chosen: " + file.getAbsolutePath());
	                         //System.out.println("File chosen (relative): " + file.getPath());
	                         URI uri = URI.createURI(file.getPath());
	                         Resource resource = resourceSet.createResource(uri);
	                         try
	     		       {
	                                 resource.load(null);
	                                 theArrangemant = (Arrangement)resource.getContents().get(0); 
	                                 //System.out.println("loaded: " + theArrangemant);
	                                 //System.out.println("The arrangement name : "+ theArrangemant.getArrangementName() );
	                                 jTextFieldArrangementName.setText(theArrangemant.getArrangementName());
	                                 EList<StateMachine> theMachines = theArrangemant.getStateMachine();
	           
	                                String integerSensorVariables = "Integer Sensor Variables:";
	                                jLabelIntegerSensorVariables = new javax.swing.JLabel();
	                        		jLabelIntegerSensorVariables.setText(integerSensorVariables);
	                        		JPanelIntegerSensorVariables = new javax.swing.JPanel();
	                        		JPanelIntegerSensorVariables.add(jLabelIntegerSensorVariables);
	                                 
	                        		ListOfVariables theSensorVariables = (ListOfVariables) theArrangemant.getSensorVariables();
	                        		if (null!=theSensorVariables) {
	                        		//System.out.println("List of Integer sensor variables size "+ theSensorVariables.getClass().toString());
	                        		EList<IntegerVariable> theIntegerSensorVariables = theSensorVariables.getVariables();
	                        		
	                        		jLabelIntegerSensorVariablesNames = new javax.swing.JLabel[theIntegerSensorVariables.size()];
	                        		JTextFieldIntegerSensorVariablesValues = new javax.swing.JTextField[theIntegerSensorVariables.size()];
	                        		
	                        		int indexIntegerSensorVariable=0;
	                        		for (Iterator v = theIntegerSensorVariables.iterator(); v.hasNext(); ) {
	                        			IntegerVariable aVariable = (IntegerVariable)  v.next();
	                        			jLabelIntegerSensorVariablesNames[indexIntegerSensorVariable]=new javax.swing.JLabel();
	                        			jLabelIntegerSensorVariablesNames[indexIntegerSensorVariable].setText(aVariable.getVariableID()+":[0,"+aVariable.getMaximumValue()+"] ");
	                        			JTextFieldIntegerSensorVariablesValues[indexIntegerSensorVariable] = new javax.swing.JTextField();
	                        			JTextFieldIntegerSensorVariablesValues[indexIntegerSensorVariable].setText("undefined");
	                        			JPanelIntegerSensorVariables.add(jLabelIntegerSensorVariablesNames[indexIntegerSensorVariable]);
	                        			JPanelIntegerSensorVariables.add(JTextFieldIntegerSensorVariablesValues[indexIntegerSensorVariable]);
	                        			indexIntegerSensorVariable++;
	                        		}
	                        		}// build Integer sensor variables	                        		
	                        		
	                                String booleanSensorVariables = "Boolean Sensor Variables:";
	                                jLabelBooleanSensorVariables = new javax.swing.JLabel();
                       	jLabelBooleanSensorVariables.setText(booleanSensorVariables);
                       	JPanelBooleanSensorVariables = new javax.swing.JPanel();
                       	JPanelBooleanSensorVariables.add(jLabelBooleanSensorVariables);
	                                 
	                        		ListOfBooleanVariables theBooleanSensorVariables = (ListOfBooleanVariables) theArrangemant.getBooleanSensorVariables();
	                        		if (null!=theBooleanSensorVariables) {
	                        		//System.out.println("List of Boolean sensor variables size "+ theBooleanSensorVariables.getClass().toString());
	                        		EList<BooleanVariable> theListOfBooleanSensorVariables = theBooleanSensorVariables.getVariables();
	                        		jLabelBooleanSensorVariablesNames = new javax.swing.JLabel[theListOfBooleanSensorVariables.size()];
	                        		JTextFieldBooleanSensorVariablesValues = new javax.swing.JTextField[theListOfBooleanSensorVariables.size()];
	                        		
	                        		int indexBooleanrSensorVariable=0;
	                        		for (Iterator v = theListOfBooleanSensorVariables.iterator(); v.hasNext(); ) {
	                        		     BooleanVariable aVariable = (BooleanVariable)  v.next();
	                     
	                        		     jLabelBooleanSensorVariablesNames[indexBooleanrSensorVariable]=new javax.swing.JLabel();
	                        		     jLabelBooleanSensorVariablesNames[indexBooleanrSensorVariable].setText(aVariable.getVariableID());
	                        		     JTextFieldBooleanSensorVariablesValues[indexBooleanrSensorVariable] = new javax.swing.JTextField();
	                        		     JTextFieldBooleanSensorVariablesValues[indexBooleanrSensorVariable].setText("undefined");
	                        		     JPanelBooleanSensorVariables.add(jLabelBooleanSensorVariablesNames[indexBooleanrSensorVariable]);
	                        		     JPanelBooleanSensorVariables.add(JTextFieldBooleanSensorVariablesValues[indexBooleanrSensorVariable]);
	                        		     indexBooleanrSensorVariable++;
	                        		}
	                        		}// build Boolean sensor variables

	                        		
	                                String integerEffectorVariables = "Integer Effector Variables:";
	                            	jLabelIntegerExternalVariables = new javax.swing.JLabel();
	                        		jLabelIntegerExternalVariables.setText(integerEffectorVariables);
	                        		JPanelIntegerExternalVariables = new javax.swing.JPanel();
	                        		JPanelIntegerExternalVariables.add(jLabelIntegerExternalVariables);
	                                 
	                        		ListOfVariables theEffectorVariables = (ListOfVariables) theArrangemant.getEffectorVariables();
	                        		if (null!=theEffectorVariables) {
	                        		//System.out.println("List of Integer effector variables size "+ theEffectorVariables.getClass().toString());
	                        		EList<IntegerVariable> theIntegerEffectorVariables = theEffectorVariables.getVariables();
	                        		jLabelIntegerExternalVariablesNames = new javax.swing.JLabel[theIntegerEffectorVariables.size()];
	                        		JTextFieldIntegerExternalVariablesValues = new javax.swing.JTextField[theIntegerEffectorVariables.size()];
	                        		
	                        		int indexIntegerExternalVariable=0;
	                        		for (Iterator v = theIntegerEffectorVariables.iterator(); v.hasNext(); ) {
	                        			IntegerVariable aVariable = (IntegerVariable)  v.next();
	                        			jLabelIntegerExternalVariablesNames[indexIntegerExternalVariable]=new javax.swing.JLabel();
	                        			jLabelIntegerExternalVariablesNames[indexIntegerExternalVariable].setText(aVariable.getVariableID()+":[0,"+aVariable.getMaximumValue()+"] ");
	                        			JTextFieldIntegerExternalVariablesValues[indexIntegerExternalVariable] = new javax.swing.JTextField();
	                        			JTextFieldIntegerExternalVariablesValues[indexIntegerExternalVariable].setText("undefined");
	                        			JPanelIntegerExternalVariables.add(jLabelIntegerExternalVariablesNames[indexIntegerExternalVariable]);
	                        			JPanelIntegerExternalVariables.add(JTextFieldIntegerExternalVariablesValues[indexIntegerExternalVariable]);
	                        			indexIntegerExternalVariable++;
	                        		}
	                        		}// build Integer effector variables
	                        		
	                        	
	                        		
	                                 String booleanEffectorVariables = "Boolean Effector Variables:";
	                                 jLabelBooleanExternalVariables = new javax.swing.JLabel();
	                                 jLabelBooleanExternalVariables.setText(booleanEffectorVariables);
                       	JPanelBooleanExternalVariables = new javax.swing.JPanel();
                       	JPanelBooleanExternalVariables.add(jLabelBooleanExternalVariables);
	                                 
	                        		ListOfBooleanVariables theBooleanEffectorVariables = (ListOfBooleanVariables) theArrangemant.getBooleanEffectorVariables();
	                        		if (null!=theBooleanEffectorVariables) {
	                        		//System.out.println("List of Boolean effector variables size "+ theBooleanEffectorVariables.getClass().toString());
	                        		EList<BooleanVariable> theListOfBooleanEffectorVariables = theBooleanEffectorVariables.getVariables();
	                        		jLabelBooleanExternalVariablesNames = new javax.swing.JLabel[theListOfBooleanEffectorVariables.size()];
	                        		JTextFieldBooleanExternalVariablesValues = new javax.swing.JTextField[theListOfBooleanEffectorVariables.size()];
	                        		
	                        		int indexBooleanExternalVariable=0;
	                        		for (Iterator v = theListOfBooleanEffectorVariables.iterator(); v.hasNext(); ) {
	                        		     BooleanVariable aVariable = (BooleanVariable)  v.next();
	                        		     jLabelBooleanExternalVariablesNames[indexBooleanExternalVariable]=new javax.swing.JLabel();
	                        		     jLabelBooleanExternalVariablesNames[indexBooleanExternalVariable].setText(aVariable.getVariableID());
	                        		     JTextFieldBooleanExternalVariablesValues[indexBooleanExternalVariable] = new javax.swing.JTextField();
	                        		     JTextFieldBooleanExternalVariablesValues[indexBooleanExternalVariable].setText("undefined");
	                        		     JPanelBooleanExternalVariables.add(jLabelBooleanExternalVariablesNames[indexBooleanExternalVariable]);
	                        		     JPanelBooleanExternalVariables.add(JTextFieldBooleanExternalVariablesValues[indexBooleanExternalVariable]);
	                        		     indexBooleanExternalVariable++;
	                        		}
	                        		}// build Boolean effector variables
	                        		

	                        		
	                        		// Shared WHITEBOARD variables
	                                String integerWhiteboardVariables = "Integer Whiteboard Variables:";
	                                jLabelIntegerWhiteboardVariables = new javax.swing.JLabel();
	                        		jLabelIntegerWhiteboardVariables.setText(integerWhiteboardVariables);
	                        		JPanelIntegerWhiteboardVariables = new javax.swing.JPanel();
	                        		JPanelIntegerWhiteboardVariables.add(jLabelIntegerWhiteboardVariables);
	                        		
	                        		ListOfVariables theWhiteboardVariables = (ListOfVariables) theArrangemant.getWhiteboardVariables();
	                        		if (null!=theWhiteboardVariables) {
	                        		//System.out.println("List of Integer whiteboard variables size "+ theWhiteboardVariables.getClass().toString());
	                        		EList<IntegerVariable> theIntegerWhiteboardVariables = theWhiteboardVariables.getVariables();
	                        		jLabelIntegerWhiteboardVariablesNames = new javax.swing.JLabel[theIntegerWhiteboardVariables.size()];
	                        		JTextFieldIntegerWhiteboardVariablesValues = new javax.swing.JTextField[theIntegerWhiteboardVariables.size()];
	                        		
	                        		int indexIntegerWhiteboardVariable=0;
	                        		for (Iterator v = theIntegerWhiteboardVariables.iterator(); v.hasNext(); ) {
	                        			IntegerVariable aVariable = (IntegerVariable)  v.next();
	                        			jLabelIntegerWhiteboardVariablesNames[indexIntegerWhiteboardVariable]=new javax.swing.JLabel();
	                        			jLabelIntegerWhiteboardVariablesNames[indexIntegerWhiteboardVariable].setText(aVariable.getVariableID()+":[0,"+aVariable.getMaximumValue()+"] ");
	                        			JTextFieldIntegerWhiteboardVariablesValues[indexIntegerWhiteboardVariable] = new javax.swing.JTextField();
	                        			JTextFieldIntegerWhiteboardVariablesValues[indexIntegerWhiteboardVariable].setText("undefined");
	                        			JPanelIntegerWhiteboardVariables.add(jLabelIntegerWhiteboardVariablesNames[indexIntegerWhiteboardVariable]);
	                        			JPanelIntegerWhiteboardVariables.add(JTextFieldIntegerWhiteboardVariablesValues[indexIntegerWhiteboardVariable]);
	                        			indexIntegerWhiteboardVariable++;
	                        		}
	                        		}// build Integer effector variables
	                        		
	                        		
	                                 String booleanWhiteboardVariables = "Boolean Whiteboard Variables:";
	                                 jLabelBooleanWhiteboardVariables = new javax.swing.JLabel();
                       	jLabelBooleanWhiteboardVariables.setText(booleanWhiteboardVariables);
                       	JPanelBooleanWhiteboardVariables = new javax.swing.JPanel();
                       	JPanelBooleanWhiteboardVariables.add(jLabelBooleanWhiteboardVariables);
                       		
	                        		ListOfBooleanVariables theBooleanWhiteboardVariables = (ListOfBooleanVariables) theArrangemant.getBooleanWhiteboardVariables();
	                        		if (null!=theBooleanWhiteboardVariables) {
	                        		//System.out.println("List of Boolean effector variables size "+ theBooleanWhiteboardVariables.getClass().toString());
	                        		EList<BooleanVariable> theListOfBooleanWhiteboardVariables = theBooleanWhiteboardVariables.getVariables();
	                        		jLabelBooleanWhiteboardVariablesNames = new javax.swing.JLabel[theListOfBooleanWhiteboardVariables.size()];
	                        		JTextFieldBooleanWhiteboardVariablesValues = new javax.swing.JTextField[theListOfBooleanWhiteboardVariables.size()];
	                        		
	                        		int indexBooleanWhiteboardVariable=0;
	                        		for (Iterator v = theListOfBooleanWhiteboardVariables.iterator(); v.hasNext(); ) {
	                        		     BooleanVariable aVariable = (BooleanVariable)  v.next();
	                        		     jLabelBooleanWhiteboardVariablesNames[indexBooleanWhiteboardVariable]=new javax.swing.JLabel();
	                        		     jLabelBooleanWhiteboardVariablesNames[indexBooleanWhiteboardVariable].setText(aVariable.getVariableID());
	                        		     JTextFieldBooleanWhiteboardVariablesValues[indexBooleanWhiteboardVariable] = new javax.swing.JTextField();
	                        		     JTextFieldBooleanWhiteboardVariablesValues[indexBooleanWhiteboardVariable].setText("undefined");
	                        		     JPanelBooleanWhiteboardVariables.add(jLabelBooleanWhiteboardVariablesNames[indexBooleanWhiteboardVariable]);
	                        		     JPanelBooleanWhiteboardVariables.add(JTextFieldBooleanWhiteboardVariablesValues[indexBooleanWhiteboardVariable]);
	                        		     indexBooleanWhiteboardVariable++;
	                        		}
	                        		}// build Boolean effector variables
	                        		
	                        		
	                        		
	                        		
	                        		
	                        		
	                                 if (null != theMachines) {
	                                     //System.out.println("The arrangement size (#machine) : "+ theMachines.size());
	                                     
	                                	 // need to add timers TODO
	                                     totalPredicatesAfter = findTotalPredciatedAfter(theMachines);
	                                     System.err.println("The arraggement has predicates after : "+ totalPredicatesAfter);
	                
	                                     
	                                     jPanelMachines = new javax.swing.JPanel[theMachines.size() + totalPredicatesAfter ];
	                                     jLabelMachineNames = new javax.swing.JLabel[theMachines.size() + totalPredicatesAfter];
	                                     jPanelMachinesTurnAndNameInfo = new javax.swing.JPanel[theMachines.size() + totalPredicatesAfter];
	                                     jLabelMachineLabelTurnNumber = new javax.swing.JLabel[theMachines.size() + totalPredicatesAfter];
	                                     jPanelVisualMachines = new VisualPanelForMachine[theMachines.size()+ totalPredicatesAfter];
	                                     jLabelIntegerLocalVariablesPanel = new javax.swing.JPanel[theMachines.size()+ totalPredicatesAfter];
	                                     jLabelBooleanLocalVariablesPanel = new javax.swing.JPanel[theMachines.size()+ totalPredicatesAfter];
	                                     
	                                     jLabelIntegerLocalVariablesNames = new ArrayList[theMachines.size()+ totalPredicatesAfter];
	                                     JTextFieldIntegerLocalVariablesValues = new ArrayList[theMachines.size()+ totalPredicatesAfter];
	                                     jLabelBooleanLocalVariablesNames = new ArrayList[theMachines.size()+ totalPredicatesAfter];
	                                     JTextFieldBooleanLocalVariablesValues = new ArrayList[theMachines.size()+ totalPredicatesAfter];
	                                     jTextFieldMachineTurnNumber = new javax.swing.JTextField[theMachines.size()+ totalPredicatesAfter];
	                                     
	                                     for (int i =0 ; i <theMachines.size()+ totalPredicatesAfter; i++)
	                                     {  
	                                    	 jTextFieldMachineTurnNumber[i] = new javax.swing.JTextField();
	                                         jLabelMachineLabelTurnNumber[i]  = new javax.swing.JLabel();
	                                    	 jPanelMachines[i] = new javax.swing.JPanel();
	                                    	 jLabelMachineNames[i] = new javax.swing.JLabel();
	                                    	 jPanelMachinesTurnAndNameInfo[i] = new javax.swing.JPanel();
	                                          System.err.println("Set up machine i= "+ i);

	                                    	 jPanelVisualMachines[i] = new VisualPanelForMachine();
	                                    	 jLabelIntegerLocalVariablesPanel[i]=new javax.swing.JPanel();
	                                    	 jLabelIntegerLocalVariablesPanel[i].setLayout(
	                                    			  new BoxLayout(jLabelIntegerLocalVariablesPanel[i] , BoxLayout.X_AXIS));
	                                    	 jLabelBooleanLocalVariablesPanel[i]=new javax.swing.JPanel();
	                                    	 jLabelBooleanLocalVariablesPanel[i].setLayout(  
	                                    			 new BoxLayout(jLabelBooleanLocalVariablesPanel[i],BoxLayout.X_AXIS) );
	                                     }// for to set up dynamically the widgets
	                                     
	                                     //System.out.println("Arrays constructed : "+ theMachines.size());
	                                     
	                                     int turnNumber = 0;
	                                     for (Iterator m = theMachines.iterator(); m.hasNext(); ) {
	                                    	 StateMachine aMachine = (StateMachine)  m.next();
	                                    	 int xDimension;
	                                    	 
	                                    	 jTextFieldMachineTurnNumber[turnNumber].setText(" "+turnNumber);
	                                    	 jLabelMachineLabelTurnNumber[turnNumber].setText("turn:");
	                                    	 jLabelMachineNames[turnNumber].setText(aMachine.getName() );
	                                    	 jPanelMachinesTurnAndNameInfo[turnNumber].add(jLabelMachineLabelTurnNumber[turnNumber]);
	                                    	 jPanelMachinesTurnAndNameInfo[turnNumber].add(jTextFieldMachineTurnNumber[turnNumber]);
	                                    	 jPanelMachinesTurnAndNameInfo[turnNumber].add(jLabelMachineNames[turnNumber]);
	                                    	 
	                                         //System.out.println(turnNumber+ " Machine Name : "+ aMachine.getName() );
	                                         // A panel of local variables
	                                         ListOfVariables theLocalIntegerVariables = aMachine.getLocalVariables();
	                                         if ((null!=theLocalIntegerVariables)
	                                            || (findTotalPredciatedAfter  ( aMachine  ) >0 )) {
	                                        		jLabelIntegerLocalVariablesNames[turnNumber] = new ArrayList<javax.swing.JLabel>();
        	                        		JTextFieldIntegerLocalVariablesValues[turnNumber] = new ArrayList<javax.swing.JTextField>();
	                                         }
	                                         if (null!=theLocalIntegerVariables) {
	         	                        		//System.out.println("List of Integer local variables size "+ theLocalIntegerVariables.getClass().toString());
	         	                        		EList<IntegerVariable> theListLocalIntegerVariables = theLocalIntegerVariables.getVariables();
	         	                        		int indexIntegerLocalVariable=0;
	         	                        		for (Iterator v = theListLocalIntegerVariables.iterator(); v.hasNext(); ) {
	         	                        			IntegerVariable aVariable = (IntegerVariable)  v.next();
	         	                        			jLabelIntegerLocalVariablesNames[turnNumber].add(new javax.swing.JLabel());
	         	                        			jLabelIntegerLocalVariablesNames[turnNumber].get(indexIntegerLocalVariable).setText(aVariable.getVariableID()+":[0,"+aVariable.getMaximumValue()+"] ");
	         	                        			JTextFieldIntegerLocalVariablesValues[turnNumber].add( new javax.swing.JTextField() );
	         	                        			JTextFieldIntegerLocalVariablesValues[turnNumber].get(indexIntegerLocalVariable).setText("undefined");
	         	                        			jLabelIntegerLocalVariablesPanel[turnNumber].add(jLabelIntegerLocalVariablesNames[turnNumber].get(indexIntegerLocalVariable));
	         	                        			jLabelIntegerLocalVariablesPanel[turnNumber].add(JTextFieldIntegerLocalVariablesValues[turnNumber].get(indexIntegerLocalVariable));
	         	                        			indexIntegerLocalVariable++;
	         	                        		}
	                                         }
	                                         // INTEGER local variables because of AFTER
	                                         if (findTotalPredciatedAfter  ( aMachine  ) >0 ) {
	                                        	// need to find the list of states of the machine that are source of transition with AFTER
	                                        	 EList<Transition> theListOfTransitions = aMachine.getTransitions();
	                                        	 boolean firstTime = true;
	                                        	 int transitionCounter =1;
	                                     for (Iterator t = theListOfTransitions.iterator(); t.hasNext(); ) {
	                                    	 Transition aTransition = (Transition)  t.next();
	                                    	 if (findTotalPredciatedAfter(aTransition)>0) {
	                                    		 
	                                    	   if (firstTime) {
	                                          	 jLabelIntegerLocalVariablesNames[turnNumber].add(new javax.swing.JLabel());
	         	                        		 JTextFieldIntegerLocalVariablesValues[turnNumber].add( new javax.swing.JTextField() );
	                                        	 int lastIndex = jLabelIntegerLocalVariablesNames[turnNumber].size()-1;
	         	                        	     jLabelIntegerLocalVariablesNames[turnNumber].get(lastIndex).setText(aMachine.getName()+"_"+aTransition.getSourceState().getStateName()+"_STEP"+":[0,"+"1"+"] ");
	         	                        		 JTextFieldIntegerLocalVariablesValues[turnNumber].get(lastIndex).setText("undefined");

	         	                        	     jLabelIntegerLocalVariablesPanel[turnNumber].add(jLabelIntegerLocalVariablesNames[turnNumber].get(lastIndex));
	         	                        	     jLabelIntegerLocalVariablesPanel[turnNumber].add(JTextFieldIntegerLocalVariablesValues[turnNumber].get(lastIndex));
	         	                        	    firstTime=false;
	                                    	   } // per state
	                                    	   // BOUNDS are per transition
	                                    	   jLabelIntegerLocalVariablesNames[turnNumber].add(new javax.swing.JLabel());
	         	                        	   JTextFieldIntegerLocalVariablesValues[turnNumber].add( new javax.swing.JTextField() );
	         	                        	   int lastIndex = jLabelIntegerLocalVariablesNames[turnNumber].size()-1;
	         	                        	   jLabelIntegerLocalVariablesNames[turnNumber].get(lastIndex).setText(aMachine.getName()+"_"+aTransition.getSourceState().getStateName()+"_BOUND_"+transitionCounter+":[0,"+"1"+"] ");
	         	                        	   JTextFieldIntegerLocalVariablesValues[turnNumber].get(lastIndex).setText("undefined");
	         	                        	   jLabelIntegerLocalVariablesPanel[turnNumber].add(jLabelIntegerLocalVariablesNames[turnNumber].get(lastIndex));
	         	                        	     jLabelIntegerLocalVariablesPanel[turnNumber].add(JTextFieldIntegerLocalVariablesValues[turnNumber].get(lastIndex));
	         	                        	    transitionCounter++;
	                                    	 }// transition has AFTER
	                                    	
	                                     }// iterate transition
	                                         }// Integer variables because this machine has AFTER
	                                         
	                                         ListOfBooleanVariables theLocalBooleanvariables = aMachine.getBooleanLocalVariables();
	                                         if ((null!=theLocalBooleanvariables)
	 	                                            || (findTotalPredciatedAfter  ( aMachine  ) >0 )) {
	                                        	    jLabelBooleanLocalVariablesNames[turnNumber] = new ArrayList<javax.swing.JLabel>();
        	                        		JTextFieldBooleanLocalVariablesValues[turnNumber] = new ArrayList<javax.swing.JTextField>();
	 	                                         }
	                                         if (null!=theLocalBooleanvariables) {
        	                        		EList<BooleanVariable> theListLocalBooleanVariables = theLocalBooleanvariables.getVariables();
        	                        		int indexBooleanLocalVariable=0;
        	                        		for (Iterator v = theListLocalBooleanVariables.iterator(); v.hasNext(); ) {
        	                        			BooleanVariable aVariable = (BooleanVariable) v.next();
        	                        			jLabelBooleanLocalVariablesNames[turnNumber].add(new javax.swing.JLabel());
        	                        			jLabelBooleanLocalVariablesNames[turnNumber].get(indexBooleanLocalVariable).setText(" bool:"+aVariable.getVariableID());
        	                        			JTextFieldBooleanLocalVariablesValues[turnNumber].add( new javax.swing.JTextField() );
        	                        			JTextFieldBooleanLocalVariablesValues[turnNumber].get(indexBooleanLocalVariable).setText("undefined");
        	                        			jLabelBooleanLocalVariablesPanel[turnNumber].add(jLabelBooleanLocalVariablesNames[turnNumber].get(indexBooleanLocalVariable));
        	                        			jLabelBooleanLocalVariablesPanel[turnNumber].add(JTextFieldBooleanLocalVariablesValues[turnNumber].get(indexBooleanLocalVariable));
        	                        			indexBooleanLocalVariable++;
        	                        		}

	                                         }
	                                      // BOOLEAN local variables because of AFTER
	                                         if (findTotalPredciatedAfter  ( aMachine  ) >0 ) {
	                                        	 // need to find the list of states of the machine that are source of transition with AFTER
	                                        	 EList<Transition> aListTransitions = aMachine.getTransitions();
	                                     for (Iterator t = aListTransitions.iterator(); t.hasNext(); ) {
	                                    	 Transition aTransition = (Transition)  t.next();
	                                    	 if (findTotalPredciatedAfter(aTransition)>0) {
	                                    		 jLabelBooleanLocalVariablesNames[turnNumber].add(new javax.swing.JLabel());
	                                    		 JTextFieldBooleanLocalVariablesValues[turnNumber].add( new javax.swing.JTextField() );
	                                    		 int lastIndex = jLabelBooleanLocalVariablesNames[turnNumber].size()-1;
	                                    		 jLabelBooleanLocalVariablesNames[turnNumber].get(lastIndex).setText("bool:"+aMachine.getName()+"_"+aTransition.getSourceState().getStateName()+"_ACTIVE");
	                                    		 JTextFieldBooleanLocalVariablesValues[turnNumber].get(lastIndex).setText("undefined");
	                                        	 
	                                    		 jLabelBooleanLocalVariablesPanel[turnNumber].add(jLabelBooleanLocalVariablesNames[turnNumber].get(lastIndex));
	                                    		 jLabelBooleanLocalVariablesPanel[turnNumber].add(JTextFieldBooleanLocalVariablesValues[turnNumber].get(lastIndex));
	                                    	 } // we have AFTER
	                                     }// iterate transition
	                                         }// boolean local variables because after
	                                         
	                                         
	                                         // the drawing of states and transitions
	                                         buildVisualPanelForMachineWithMachine(jPanelVisualMachines[turnNumber],aMachine,turnNumber);
	                                 
                                    	 
                                    pack();
                                    xDimension = 250+5*aMachine.getStates().size();
                                    System.err.println(aMachine.getName()+"Xdimension"+xDimension);
                                    System.err.println(aMachine.getName()+"Integer dimension"+jLabelIntegerLocalVariablesPanel[turnNumber].getWidth());
                                    System.err.println(aMachine.getName()+"Boolean dimension"+jLabelBooleanLocalVariablesPanel[turnNumber].getWidth());
	                                         if (jLabelIntegerLocalVariablesPanel[turnNumber].getWidth()>xDimension)
	                                        	 {System.err.println("Changing x dimension Integer larger than"+xDimension);
	                                        	   xDimension = jLabelIntegerLocalVariablesPanel[turnNumber].getWidth();
	                                        	 }
	                                        	 
	                                         if (jLabelBooleanLocalVariablesPanel[turnNumber].getWidth()>xDimension)
	                                        	 {System.err.println("Changing x dimension Boolean larger than"+xDimension);
	                                        	 xDimension = jLabelBooleanLocalVariablesPanel[turnNumber].getWidth();
	                                        	  
	                                        	 }
                                    // as large as the widest local variable
                                    jPanelVisualMachines[turnNumber].setPreferredSize(new 
                       	        		 Dimension(xDimension, 300+20*aMachine.getStates().size() ));	                                     
                                    turnNumber++;
	                                     } // 
	                                     
	                                     // fill each machine
	                                     GridBagConstraints c = new GridBagConstraints();
	                                     for (int i =0 ; i <theMachines.size(); i++)
	                                     {
	                                    	 jPanelMachines[i].setLayout(new GridBagLayout() );
	                                    	 c.fill = GridBagConstraints.NONE;
                       	         c.gridx = 0; c.gridy = 0;
                       	         jPanelMachines[i].add(jPanelMachinesTurnAndNameInfo[i],c);
                       	         c.fill = GridBagConstraints.NONE;
                       	         c.gridx = 0; c.gridy = 1;
                       	         jPanelMachines[i].add(jLabelIntegerLocalVariablesPanel[i],c);
                       	         c.fill = GridBagConstraints.NONE;
                       	         c.gridx = 0; c.gridy = 2;
                       	         jPanelMachines[i].add(jLabelBooleanLocalVariablesPanel[i],c);
                       	         c.fill = GridBagConstraints.NONE;
                       	         c.gridx = 0; c.gridy = 3;
                       	         jPanelMachines[i].add(jPanelVisualMachines[i],c);
                       	         
                       	         // TODO add layout button
                       	         jPanelVisualMachines[i].AutoLayout();
	                                     }
	                                     
	                                     // If we require timers we build them here
	                                     if (totalPredicatesAfter>0 ) {
	                                    	 // Maybe easier to iterate over machines and then over their transitions
	                                    	 int theNewTurn;;
	                                    	 for (int newLabel=0; newLabel< totalPredicatesAfter; newLabel++ ) {
	                                    		 // initialise the list for this
	                                    		 theNewTurn= theMachines.size()+ newLabel;
	                                    		 jLabelIntegerLocalVariablesNames[theNewTurn] = new ArrayList<javax.swing.JLabel>();
	                                    		 JTextFieldIntegerLocalVariablesValues[theNewTurn] = new ArrayList<javax.swing.JTextField>();
	                                    		 jLabelBooleanLocalVariablesNames[theNewTurn] = new ArrayList<javax.swing.JLabel>();
	                                    		 JTextFieldBooleanLocalVariablesValues[theNewTurn] = new ArrayList<javax.swing.JTextField>();
	                                    	 }
	                                    	 
	                                    	 int machineNumber = 0;
	                                    	 int transitionsFoundWithAfter=0;
                                    for (Iterator m = theMachines.iterator(); m.hasNext(); ) {
                                   	 StateMachine aMachine = (StateMachine)  m.next();
                                   	 EList<Transition> theListOfTransitions = aMachine.getTransitions();
                                   	   
                                   	   int transitionNumberinMachine=1; // start from one because of the dummy transition
                                   	   for (Iterator t = theListOfTransitions.iterator(); t.hasNext(); ) {
                                   	 Transition aTransition = (Transition)  t.next();
                                   	 if (findTotalPredciatedAfter(aTransition)>0) {
                                   		 theNewTurn = theMachines.size()+transitionsFoundWithAfter;
                                   		 jTextFieldMachineTurnNumber[theNewTurn].setText(" "+theNewTurn);
                                   		 jLabelMachineLabelTurnNumber[theNewTurn].setText("turn:");
                                   		 jLabelMachineNames[theNewTurn].setText("Timer_"
                                   				 +aMachine.getName()+"_"
                                   				 +aTransition.getSourceState().getStateName()+"_"
                                   				 +aTransition.getTargetState().getStateName()+"_"+transitionNumberinMachine
                                   				 );
                                   		 jPanelMachinesTurnAndNameInfo[theNewTurn].add(jLabelMachineLabelTurnNumber[theNewTurn]);
	                                    	 jPanelMachinesTurnAndNameInfo[theNewTurn].add(jTextFieldMachineTurnNumber[theNewTurn]);
	                                    	 jPanelMachinesTurnAndNameInfo[theNewTurn].add(jLabelMachineNames[theNewTurn]);
	                                    	 // We need the integer variables LocalCurrentTime
	                                    	 
	                                    	 jLabelIntegerLocalVariablesNames[theNewTurn].add(new javax.swing.JLabel());
	                                    	 jLabelIntegerLocalVariablesNames[theNewTurn].get(0).setText("LocalCurrentTime"+":[0,"+
	                                    			 getBoundInAfter(  aTransition.getBooleanExpression()  )  +"]");
	                                    	     
	                                    	 JTextFieldIntegerLocalVariablesValues[theNewTurn].add( new javax.swing.JTextField() );
	                                    	 JTextFieldIntegerLocalVariablesValues[theNewTurn].get(0).setText("undefined");
	                                    	 jLabelIntegerLocalVariablesPanel[theNewTurn].add(jLabelIntegerLocalVariablesNames[theNewTurn].get(0));
	                                    	 jLabelIntegerLocalVariablesPanel[theNewTurn].add(JTextFieldIntegerLocalVariablesValues[theNewTurn].get(0));
	                                    	 
	                                    	 // We need the boolean variables finished
	                                    	 jLabelBooleanLocalVariablesNames[theNewTurn].add(new javax.swing.JLabel());
	                                    	 jLabelBooleanLocalVariablesNames[theNewTurn].get(0).setText("bool:finished");
	                                    	 JTextFieldBooleanLocalVariablesValues[theNewTurn].add( new javax.swing.JTextField() );
	                                    	 JTextFieldBooleanLocalVariablesValues[theNewTurn].get(0).setText("undefined");
	                                    	 jLabelBooleanLocalVariablesPanel[theNewTurn].add(jLabelBooleanLocalVariablesNames[theNewTurn].get(0));
	                                    	 jLabelBooleanLocalVariablesPanel[theNewTurn].add(JTextFieldBooleanLocalVariablesValues[theNewTurn].get(0));
	                                    	 
	                                    	 // are we drawing it
	                                    	 jPanelMachines[theNewTurn].setLayout(new GridBagLayout() );
	                                    	 c.fill = GridBagConstraints.NONE;
		                        	         c.gridx = 0; c.gridy = 0;
		                        	         jPanelMachines[theNewTurn].add(jPanelMachinesTurnAndNameInfo[theNewTurn],c);
		                        	         
		                        	         c.fill = GridBagConstraints.NONE;
		                        	         c.gridx = 0; c.gridy = 1;
		                        	         jPanelMachines[theNewTurn].add(jLabelIntegerLocalVariablesPanel[theNewTurn],c);
		                        	         
		                        	         c.fill = GridBagConstraints.NONE;
		                        	         c.gridx = 0; c.gridy = 2;
		                        	         jPanelMachines[theNewTurn].add(jLabelBooleanLocalVariablesPanel[theNewTurn],c);
		                        	         
		                        	         c.fill = GridBagConstraints.NONE;
		                        	         c.gridx = 0; c.gridy = 3;
		                        	         // to draw
		                        	         buildVisualPanelForTimer(jPanelVisualMachines[theNewTurn],
		                        	        		 aMachine.getName(), 
		                        	        		 aTransition.getSourceState().getStateName(), 
		                        	        		 aTransition.getTargetState().getStateName(), 
		                        	        		 transitionNumberinMachine, Integer.parseInt(getBoundInAfter(  aTransition.getBooleanExpression()).trim()) , 1, theNewTurn);
		                        	         jPanelMachines[theNewTurn].add(jPanelVisualMachines[theNewTurn],c);
		                        	         /*
		                        	         jPanelVisualMachines[theNewTurn].AutoLayout();
		                        	         */
		                        	         
                                   		 
                                   		 transitionsFoundWithAfter++;
                                   	 }
                                   	 
                                   	 transitionNumberinMachine++;
                                   	   }// iterate transitions
                                   	 
                                   	 
                                   	 
                                   	 machineNumber++;
                                    } // iterate over machines
	                                    	
	                                    	 
	                                    
	                                    	 
	                                     }// we need timers
	                                     
	                                     //fill the scroll Panel
	                                     jPanelInforMachinesArea.setLayout(
	                                    		 new BoxLayout( jPanelInforMachinesArea, BoxLayout.Y_AXIS  ) );
	                                     
	                                     if (null!=theSensorVariables) {
                                   	      jPanelInforMachinesArea.add(JPanelIntegerSensorVariables);
	                                     }
	                                     if (null!=theBooleanSensorVariables) {
	                                    	 jPanelInforMachinesArea.add(JPanelBooleanSensorVariables);
	                                     } 
	                                     if (null!=theEffectorVariables) {
	                                    	 jPanelInforMachinesArea.add(JPanelIntegerExternalVariables);
	                                     }
	                                     if (null!=theBooleanEffectorVariables) {
	                                    	 jPanelInforMachinesArea.add(JPanelBooleanExternalVariables);
	                                     }
	                                     if (null!=theWhiteboardVariables) {
	                                    	 jPanelInforMachinesArea.add(JPanelIntegerWhiteboardVariables);
	                                     }
                                   	 if (null!=theBooleanWhiteboardVariables) {
                                   		 jPanelInforMachinesArea.add(JPanelBooleanWhiteboardVariables);
                                   	 }
                                   	 
	                                     
                                   	 
                                   	 //jPanelMachinesArea.setLayout(  new BoxLayout( jPanelMachinesArea, BoxLayout.X_AXIS  ));
                                   	 // adding the timers by running the index further
	                                     for (int i =0 ; i <theMachines.size()+totalPredicatesAfter; i++)
	                                     {   
	                                    	 if (i<theMachines.size()) {
	                                    	         jPanelMachinesArea.addTab("LLFSM"+
	                                    	        		 jLabelMachineNames[i].getText( )
	                                    	        		 +" ("+i+")", null, jPanelMachines[i]);
	                                    	 }
	                                    	 else
	                                    		 jPanelMachinesArea.addTab("LLFSM TIMER ("+i+")", null, jPanelMachines[i]); 
	                                     }
	                                     
	                                     //jPanelMachinesArea.setPreferredSize(new Dimension(500, 400));
	                                     //jScrollPaneView = new javax.swing.JScrollPane(jPanelMachinesArea);
	                                     jPanelDisplayVisual = new javax.swing.JPanel(); //,ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
	                                     //jScrollPaneView = new javax.swing.JScrollPane(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

	                                     jPanelDisplayVisual.setLayout(  
	                                    		 new BoxLayout( jPanelDisplayVisual, BoxLayout.Y_AXIS  ) );
	                                    		 
	                                     jPanelDisplayVisual.add(jPanelInforMachinesArea);
	                                     jPanelDisplayVisual.add(jPanelMachinesArea);
	                                     //jPanelDisplayVisual.setPreferredSize(new Dimension(800, 400));
	                                     // Add Pane/ View to Global Frame
	                                     
	                                     //jScrollPaneView.add(jPanelDisplayVisual);
	                                   
	                        	         //getContentPane().add(jScrollPaneView,"West");
	                                     getContentPane().add(jPanelDisplayVisual,"West");
	                        	    	 
	         
	                                     }
	                                
	                                 //jPanelFirstMachine.setBackground(Color.green);
	     		
	                         }
	                             catch (IOException e)
	                         {
	                             System.err.println("failed to read " + uri); 
	                             System.err.println( "Some exception");
	                             System.err.println("General I/O exception: " + e.getMessage());
	                             e.printStackTrace();
	                         }
	                         jMenuItemOpen.setEnabled(false); // can't open a new arrangement
	                         jMenuItemRedraw.setEnabled(true);
	                         jMenuExpandViewLLFSM.setEnabled(true);
	                    	 
	   	        	 	// if we have a nusmv trace (because the opening a trace is not enabled)
	                           // we should be able to emulate the trace
	   	        	 	if (! jMenuItemOpenNuSSMV.isEnabled() ) {
	   	        	 		jButtonNext.setEnabled(true);
	        	 	jMenuItemNext.setEnabled(true);
	   	        	 	}
	            

	                }
	                else
	                { userNavigating=true; // reset the file chooser
	                  OpenMachineFileChooser.setCurrentDirectory(file);
	                }
	            } else {userNavigating=false;        }   
	           } while (userNavigating);
	             
	         }  /* jMenuItemOpenActionPerformed*/    

                
/**
 * Constructor call a routine to initialize all widgets
*/
public EmulatenuXmvTracesOfLLFSms() {
                initComponents();
} /* EmulatenuXmvTracesOfLLFSms */

private void initComponents() {
  // create the widgets
    jMenuBarTop = new javax.swing.JMenuBar();
    jLabelArrangementFileName = new javax.swing.JLabel();
    jLabelNusmvFileName = new javax.swing.JLabel();
    OpenMachineFileChooser = new javax.swing.JFileChooser();
                 //OpenNuSMVTraceFileChooser = new javax.swing.JFileChooser();
    jTextFieldTraceFile = new javax.swing.JTextField();
    jTextFieldArrangementName = new javax.swing.JTextField();
    jButtonNext = new javax.swing.JButton();
    jLabelNusmvKripkeState = new javax.swing.JLabel();
    jTextFieldKripkeState = new javax.swing.JTextField();
    jTextFieldKripkeState.setPreferredSize(new Dimension(80, 25));
    
    jPanelMachinesArea = new javax.swing.JTabbedPane();
    jPanelInforMachinesArea = new javax.swing.JPanel();

    jMenuFile = new javax.swing.JMenu();
    jMenuItemOpen = new javax.swing.JMenuItem();
    jMenuItemExit = new javax.swing.JMenuItem();
    jMenuItemOpenNuSSMV = new javax.swing.JMenuItem();

    jMenuAction  = new javax.swing.JMenu();
    jMenuItemNext = new javax.swing.JMenuItem();
    jMenuItemRedraw = new javax.swing.JMenuItem();
    jMenuExpandViewLLFSM = new javax.swing.JMenuItem();

    jPanelInformation = new javax.swing.JPanel();
    jMenuFile.setText("File");
    jMenuAction.setText("Action");


    jMenuItemExit.setText("Exit");
    jMenuItemExit.addActionListener(new java.awt.event.ActionListener() {
                     public void actionPerformed(java.awt.event.ActionEvent evt) {
                         jMenuItemExitActionPerformed(evt);
                     }
                 });

    jMenuItemOpen.setText("Open Arrangement (xmi)");
    jMenuItemOpen.addActionListener(new java.awt.event.ActionListener() {
                     public void actionPerformed(java.awt.event.ActionEvent evt) {
                         jMenuItemOpenActionPerformed(evt);
                     }
                 });

    jMenuItemOpenNuSSMV.setText("Open NuSMV trace (nusmv)");
    jMenuItemOpenNuSSMV.addActionListener(new java.awt.event.ActionListener() {
                     public void actionPerformed(java.awt.event.ActionEvent evt) {
                         jMenuItemOpenNuSMVActionPerformed(evt);
                     }
                 });

    jMenuItemNext.setText("Next Kripke State in NuSMV Trace");
    jMenuItemNext.addActionListener(new java.awt.event.ActionListener() {
                     public void actionPerformed(java.awt.event.ActionEvent evt) {
                         jMenuItemNextActionPerformed(evt);
                     }
                 });

    jMenuItemNext.setEnabled(false); // becomes True once we read an arrangement and a trace

    jMenuItemRedraw.setText("Redraw Machines Space");
    jMenuItemRedraw.addActionListener(new java.awt.event.ActionListener() {
                     public void actionPerformed(java.awt.event.ActionEvent evt) {
                         jMenuItemRedrawActionPerformed(evt);
                     }
                 });

    jMenuItemRedraw.setEnabled(false); // becomes True once we read an arrangement

    jMenuExpandViewLLFSM.setText("Expand view on LLFSM");
    jMenuExpandViewLLFSM.addActionListener(new java.awt.event.ActionListener() {
                     public void actionPerformed(java.awt.event.ActionEvent evt) {
                         jMenuExpandViewLLFSMActionPerformed(evt);
                     }
                 });

    jMenuExpandViewLLFSM.setEnabled(false); // becomes True once we read an arrangement

    jMenuFile.add(jMenuItemExit);
    jMenuFile.add(jMenuItemOpen);
    jMenuFile.add(jMenuItemOpenNuSSMV);

    jMenuAction.add(jMenuItemNext);
    jMenuAction.add(jMenuItemRedraw);

    jMenuBarTop.add(jMenuFile);
    jMenuBarTop.add(jMenuAction);
    jMenuAction.add(jMenuExpandViewLLFSM);

    setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

    jLabelArrangementFileName.setText("Arrangement Name   ");
    jLabelNusmvFileName.setText("nusmv Filename   ");

    jTextFieldArrangementName.setEditable(false);
    jTextFieldArrangementName.setSize(40, 10);
    jTextFieldTraceFile.setEditable(false);
    jTextFieldTraceFile.setSize(40,10);
    jLabelNusmvKripkeState.setText("Kripke State:");
    jTextFieldKripkeState.setEditable(false);

    jButtonNext.setText("NEXT");
    jButtonNext.setEnabled(false); // same behaviour than the Menu option under Action
    jButtonNext.addActionListener(new java.awt.event.ActionListener() {
                             public void actionPerformed(java.awt.event.ActionEvent evt) {
                                 jMenuItemNextActionPerformed(evt);
                             }
                         });  
    
    this.setTitle("nusmv Trace Emulator (c) 2020 Vlad Estivill-Castro");
    this.setSize(400,100);
    this.setVisible(true);
    this.setJMenuBar(jMenuBarTop);

    jPanelInformation.setLayout(new GridBagLayout(  ) );
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.NONE;
    c.gridx = 0; c.gridy = 0;
    jPanelInformation.add(jLabelArrangementFileName,c);
    c.fill = GridBagConstraints.NONE;
    c.gridx = 1; c.gridy = 0;
    jPanelInformation.add(jTextFieldArrangementName,c);
                   c.fill = GridBagConstraints.NONE;
                   c.gridx = 2; c.gridy = 0;
                   jPanelInformation.add(jLabelNusmvFileName,c);
                  c.fill = GridBagConstraints.NONE;
                   c.gridx = 3; c.gridy = 0;
                   jPanelInformation.add(jTextFieldTraceFile,c);
                   c.fill = GridBagConstraints.NONE;
                   c.gridx = 4; c.gridy = 0;
                   jPanelInformation.add(jLabelNusmvKripkeState,c);
                   c.fill = GridBagConstraints.NONE;
                   c.gridx = 5; c.gridy = 0;
                   jPanelInformation.add(jTextFieldKripkeState,c);
                  c.fill = GridBagConstraints.NONE;
                   c.gridx = 6; c.gridy = 0;
                   jPanelInformation.add(jButtonNext,c);


    getContentPane().setLayout( new BorderLayout() );

    getContentPane().add(jPanelInformation,"North");



  
} // init components
            

    /**
     * MAIN PROGRAM
     * @param args the command line arguments
     */
   		public static void main(String[] args) {
			System.out.println(" (c) 2020 Vlad Estivill-Castro");
	        try {
	            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
	                if ("Nimbus".equals(info.getName())) {
	                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
	                    break;
	                }
	            }
	        } catch (ClassNotFoundException ex) {
	            java.util.logging.Logger.getLogger(EmulatenuXmvTracesOfLLFSms.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
	        } catch (InstantiationException ex) {
	            java.util.logging.Logger.getLogger(EmulatenuXmvTracesOfLLFSms.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
	        } catch (IllegalAccessException ex) {
	            java.util.logging.Logger.getLogger(EmulatenuXmvTracesOfLLFSms.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
	        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
	            java.util.logging.Logger.getLogger(EmulatenuXmvTracesOfLLFSms.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
	        }
			
	        /* Create and display the form */
	        java.awt.EventQueue.invokeLater(new Runnable() {
	           public void run() {
	                new EmulatenuXmvTracesOfLLFSms().setVisible(true);
	             }
	         });

		}// main
    
} /* enf of EmulatenuXmvTracesOfLLFSms */
    
